"""
Copyright (c) 2024-2025 ROCK LAB PRIVATE LIMITED
This file is part of "Stresslog" project and is released under the 
GNU Affero General Public License v3.0 (AGPL-3.0)
See the GNU Affero General Public License for more details: <https://www.gnu.org/licenses/agpl-3.0.html>
"""

import matplotlib.pyplot as plt
import numpy as np

from io import StringIO

def plot_to_svg(matplot) -> str:
    """
    Saves the last plot made using ``matplotlib.pyplot`` to a SVG string.
    
    Returns:
        The corresponding SVG string.
    """
    s = StringIO()
    matplot.savefig(s, format="svg")
    matplot.close()  # https://stackoverflow.com/a/18718162/14851404
    return s.getvalue()


def plotfracs(data):
    """
    Plot line segments representing fractures based on position and angle data.
    
    Parameters
    ----------
    data : ndarray
        A 2D numpy array with shape (n, 4) where:
        - data[:, 0] contains y-coordinates
        - data[:, 1] contains x-coordinates
        - data[:, 2] contains first set of angles
        - data[:, 3] contains second set of angles
    
    Returns
    -------
    matplotlib.pyplot
        A matplotlib pyplot object containing the plotted fractures
    """
    x = data[:, 1]
    x2 = x+180
    y = data[:, 0]
    angles = data[:, 2]
    angles2 = -data[:, 3]
    # Creating the plot
    fig, ax = plt.subplots()
    ax.set_xlim(0, 360)  # Adjusting x-axis limit to fit your data
    #ax.set_ylim(2450, 2460)  # Adjusting y-axis limit to fit your data
    #ax.set_aspect(0.2)  # This makes 1 unit in x equal to 1 unit in y

    # Plotting each line segment based on the angle
    for x_i, y_i, angle in zip(x, y, angles):
        # Calculating the end point of the line segment
        # Assuming a fixed length for each segment
        length = 1  # Adjusted length to be visible given the scale of your data
        end_x = x_i + length * np.cos(np.radians(angle))
        end_y = y_i + length * np.sin(np.radians(angle))
        
        # Plotting the line segment
        ax.plot([x_i, end_x], [y_i, end_y], linewidth=0.01, color='black')  # Added markers for clarity
        
        
    for x2_i, y_i, angle2 in zip(x2, y, angles2):
        # Calculating the end point of the line segment
        # Assuming a fixed length for each segment
        length = 1  # Adjusted length to be visible given the scale of your data
        end_x = x2_i - length * np.cos(np.radians(angle2))
        end_y = y_i - length * np.sin(np.radians(angle2))
        
        # Plotting the line segment
        ax.plot([x2_i, end_x], [y_i, end_y], linewidth=0.01, color='black')  # Added markers for clarity
        
    return plt
#plt.show()

def plotfracsQ(data):
    """
    Initialize a plot for fractures without plotting the line segments.
    
    Parameters
    ----------
    data : ndarray
        A 2D numpy array with shape (n, 4) where:
        - data[:, 0] contains y-coordinates
        - data[:, 1] contains x-coordinates
        - data[:, 2] contains first set of angles
        - data[:, 3] contains second set of angles
    
    Returns
    -------
    matplotlib.pyplot
        A matplotlib pyplot object with initialized axes set to [0, 360]
    """
    x = data[:, 1]
    x2 = x+180
    y = data[:, 0]
    angles = data[:, 2]
    angles2 = -data[:, 3]
    # Creating the plot
    fig, ax = plt.subplots()
    ax.set_xlim(0, 360)  # Adjusting x-axis limit to fit your data
    #ax.set_ylim(2450, 2460)  # Adjusting y-axis limit to fit your data
    #ax.set_aspect(0.2)  # This makes 1 unit in x equal to 1 unit in y

    return plt
#plt.show()

def cot(x):
    return np.cos(x)/np.sin(x)

def plotfrac(data,path=None, dia=8.5, debug=False):
    """
    Generate detailed fracture morphology plot with depth calculations.
    
    Parameters
    ----------
    data : tuple
        A tuple containing:
        - tvd : float
            True vertical depth
        - fr : ndarray
            Array of fracture indicators
        - angles : ndarray
            Array of angles
        - minangle : float
            Minimum angle value
        - maxangle : float
            Maximum angle value
    path : str, optional
        File path to save the plot. If None, plot is not saved to file.
    dia  : float, optional
        Hole Diameter in inches to be used for the calculation, in inches, 8.5 by default
    
    Returns
    -------
    tuple
        A tuple containing two ndarrays:
        - cdepths : ndarray
            Corrected depths array
        - fdepths : ndarray
            Final depths array
    
    Notes
    -----
    Uses a bit diameter of 8.5 inches for calculations.
    Handles special angle cases and implements various depth corrections
    and transformations.
    """
    tvd,fr,angles,minangle,maxangle = data
    #dia = 8.5 #inches, bit
    circumference = np.pi*dia #in inches
    cm = 0.0254*circumference
    i = 0
    d = np.zeros(360)
    yj = np.zeros(360)
    depths = np.zeros(360)
    sign = np.zeros(360)
    midpoint1 = min(minangle+90,(minangle+270)%360)
    midpoint2 = max(minangle+90,(minangle+270)%360)
    spVa = 360
    spVb = 0
    while(i<360):
        if i>midpoint1 and i<midpoint2:
            d[i] = (i-(minangle+180))
        else:
            if i<midpoint1:
                d[i] = i-minangle
            else:
                d[i] = i-(360+minangle)
        if abs(d[i])==270:
            d[i]=90*(abs(d[i])/d[i])
        yj[i] = (np.tan(np.radians((90-angles[i])%360))*d[i]) #FlipVertN if abs
        sign[i] = yj[i]/abs(yj[i])
        depths[i] = (((yj[i]-180)/180)*(cm/2))
        if d[i-1]==0:
            yj[i-1] = (yj[i-2]+yj[i])/2
            spV =  yj[i-1]
            depths[i-1] = (depths[i-2]+depths[i])/2
            spVa = min(depths[i-1],spVa)
            spVb = max(depths[i-1],spVb)

        i+=1
    
    yj = yj-spV
    #depths = depths*sign #Dont FlipV if signed #AntiFlipV
    if path is not None:
        plt.figure(figsize=(10, 10))
        plt.plot(yj)
        #Setting axis limits
        plt.xlim(0, 360)
        plt.ylim(-180, 180)
        plt.title("Fracture Morphology")
        plt.savefig(path)
        plt.close()
    print(yj) if debug else None
    yj[(maxangle-10)%360:(maxangle+15)%360]=np.nan
    yj[(maxangle+170)%360:(maxangle+195)%360]=np.nan
    #depths[(maxangle-10)%360:(maxangle+15)%360]=np.nan
    #depths[(maxangle+170)%360:(maxangle+195)%360]=np.nan
    i=0
    cyj = yj
    cyj[midpoint1:midpoint2] = cyj[midpoint1:midpoint2][::-1]
    avdepths = depths.copy()
    while i<360: #ShiftVert1
        #if i<midpoint1 or i>midpoint2-1:
        #    depths[i] = depths[i]-spVa
        #else:
        #    depths[i] = depths[i]-spVb
        if abs(depths[i])>1.6:
            #depths[i] = np.nan
            avdepths[i] = np.nan
        if fr[i]<1:
            depths[i] = np.nan
        i+=1
    av1 = np.nanmean(np.concatenate([depths[0:(midpoint1-1)],depths[midpoint2:360]]))
    av2 = np.nanmean(depths[midpoint1:(midpoint2-1)])
    
    i=0
    while i<360: #ShiftVert2
        if i<midpoint1 or i>midpoint2-1:
            depths[i] = depths[i]-av1
        else:
            depths[i] = depths[i]-av2
        i+=1
    
    i=0
    fdepths=depths.copy()
    print(fdepths) if debug else None
    deldep = (np.nanmean(depths)) 
    depths = depths-deldep
    fdepths = fdepths-deldep
    cdepths = depths+tvd
    while i<360:
        if abs(depths[i])>1.6:
            cdepths[i] = np.nan
        if fr[i]<1:
            cdepths[i] = np.nan
        i+=1
    fdepths = fdepths+tvd
    m1 = np.nanmax(np.concatenate([fdepths[0:(midpoint1-1)],fdepths[midpoint2:360]]))
    m2 = np.nanmax(fdepths[midpoint1:(midpoint2-1)])
    diff = m2-m1
    print(diff) if debug else None
    fdepths[0:(midpoint1-1)] = fdepths[0:(midpoint1-1)] + diff/2
    fdepths[midpoint2:360] = fdepths[midpoint2:360] + diff/2
    fdepths[midpoint1:(midpoint2-1)]=fdepths[midpoint1:(midpoint2-1)] - diff/2
    
    cdepths[0:(midpoint1-1)] = cdepths[0:(midpoint1-1)] + diff/2
    cdepths[midpoint2:360] = cdepths[midpoint2:360] + diff/2
    cdepths[midpoint1:(midpoint2-1)]=cdepths[midpoint1:(midpoint2-1)] - diff/2
    #cdepths[midpoint1:midpoint2] = cdepths[midpoint1:midpoint2][::-1]# FlipHorzR
    #fdepths[midpoint1:midpoint2] = fdepths[midpoint1:midpoint2][::-1]# FlipHorzR
    print(fdepths) if debug else None
    print(d) if debug else None
    
    """
    while i<360:
        if fr[i]>0:
            plt.scatter(i, yj[i], color='black', marker='o')
        i+=1
    """

    return cdepths,fdepths
#plt.show()



from sympy import symbols, Eq, cos, sin, sqrt, solve

# Define variables
bhp, pp, tns, nu, theta = symbols("bhp pp tns nu theta")
Sb00, Sb11, Sb22, Sb01, Sb02, Sb12 = symbols("Sb00 Sb11 Sb22 Sb01 Sb02 Sb12")

# Derived variables
deltaP = bhp - pp
sigmaT = symbols("sigmaT")  # thermal stress
tensile_strength = tns

# Expressions for Szz, Stt, and Ttz
Szz = Sb22 - 2 * nu * (Sb00 - Sb11) * cos(2 * theta) - 4 * nu * Sb01 * sin(2 * theta)
Stt = Sb00 + Sb11 - 2 * (Sb00 - Sb11) * cos(2 * theta) - 4 * Sb01 * sin(2 * theta) - deltaP - sigmaT
Ttz = 2 * (Sb12 * cos(theta) - Sb02 * sin(theta))

# Expressions for STMax and Stmin
STMax = 0.5 * (Szz + Stt + sqrt((Szz - Stt)**2 + 4 * Ttz**2))
Stmin = 0.5 * (Szz + Stt - sqrt((Szz - Stt)**2 + 4 * Ttz**2))

# Set Stmin equal to tensile_strength
equation = Eq(Stmin, tensile_strength)

# Solve for bhp
bhp_critical_solution = solve(equation, bhp)
print(bhp_critical_solution)
"""
translated to standard python:
(40.0*Sb11**2*nu*np.cos(2.0*theta)**2 - 20.0*Sb11**2*nu*np.cos(2.0*theta) - 40.0*Sb11*Sb12*nu*np.sin(2.0*theta) + 80.0*Sb11*Sb12*nu*np.sin(4.0*theta) - 80.0*Sb11*Sb22*nu*np.cos(2.0*theta)**2 - 20.0*Sb11*Sb33*np.cos(2.0*theta) + 10.0*Sb11*Sb33 - 20.0*Sb11*nu*pp*np.cos(2.0*theta) + 20.0*Sb11*nu*sigmaT*np.cos(2.0*theta) - 2.0*Sb11*nu*ucs*np.cos(2.0*theta) - 2.0*Sb11*ucs*np.cos(2.0*theta) + Sb11*ucs + 160.0*Sb12**2*nu*np.sin(2.0*theta)**2 - 40.0*Sb12*Sb22*nu*np.sin(2.0*theta) - 80.0*Sb12*Sb22*nu*np.sin(4.0*theta) - 40.0*Sb12*Sb33*np.sin(2.0*theta) - 40.0*Sb12*nu*pp*np.sin(2.0*theta) + 40.0*Sb12*nu*sigmaT*np.sin(2.0*theta) - 4.0*Sb12*nu*ucs*np.sin(2.0*theta) - 4.0*Sb12*ucs*np.sin(2.0*theta) - 40.0*Sb13**2*np.sin(theta)**2 + 40.0*Sb13*Sb23*np.sin(2.0*theta) + 40.0*Sb22**2*nu*np.cos(2.0*theta)**2 + 20.0*Sb22**2*nu*np.cos(2.0*theta) + 20.0*Sb22*Sb33*np.cos(2.0*theta) + 10.0*Sb22*Sb33 + 20.0*Sb22*nu*pp*np.cos(2.0*theta) - 20.0*Sb22*nu*sigmaT*np.cos(2.0*theta) + 2.0*Sb22*nu*ucs*np.cos(2.0*theta) + 2.0*Sb22*ucs*np.cos(2.0*theta) + Sb22*ucs - 40.0*Sb23**2*np.cos(theta)**2 + 10.0*Sb33*pp - 10.0*Sb33*sigmaT + Sb33*ucs + pp*ucs - sigmaT*ucs + 0.1*ucs**2)/(-20.0*Sb11*nu*np.cos(2.0*theta) - 40.0*Sb12*nu*np.sin(2.0*theta) + 20.0*Sb22*nu*np.cos(2.0*theta) + 10.0*Sb33 + ucs)
"""
"""
reformulated solution:
[(-4.0*Sb00**2*nu*cos(2.0*theta)**2 + 2.0*Sb00**2*nu*cos(2.0*theta) + 4.0*Sb00*Sb01*nu*sin(2.0*theta) - 8.0*Sb00*Sb01*nu*sin(4.0*theta) + 8.0*Sb00*Sb11*nu*cos(2.0*theta)**2 + 2.0*Sb00*Sb22*cos(2.0*theta) - Sb00*Sb22 + 2.0*Sb00*nu*pp*cos(2.0*theta) - 2.0*Sb00*nu*sigmaT*cos(2.0*theta) - 2.0*Sb00*nu*tns*cos(2.0*theta) - 2.0*Sb00*tns*cos(2.0*theta) + Sb00*tns - 16.0*Sb01**2*nu*sin(2.0*theta)**2 + 4.0*Sb01*Sb11*nu*sin(2.0*theta) + 8.0*Sb01*Sb11*nu*sin(4.0*theta) + 4.0*Sb01*Sb22*sin(2.0*theta) + 4.0*Sb01*nu*pp*sin(2.0*theta) - 4.0*Sb01*nu*sigmaT*sin(2.0*theta) - 4.0*Sb01*nu*tns*sin(2.0*theta) - 4.0*Sb01*tns*sin(2.0*theta) + 4.0*Sb02**2*sin(theta)**2 - 4.0*Sb02*Sb12*sin(2.0*theta) - 4.0*Sb11**2*nu*cos(2.0*theta)**2 - 2.0*Sb11**2*nu*cos(2.0*theta) - 2.0*Sb11*Sb22*cos(2.0*theta) - Sb11*Sb22 - 2.0*Sb11*nu*pp*cos(2.0*theta) + 2.0*Sb11*nu*sigmaT*cos(2.0*theta) + 2.0*Sb11*nu*tns*cos(2.0*theta) + 2.0*Sb11*tns*cos(2.0*theta) + Sb11*tns + 4.0*Sb12**2*cos(theta)**2 - Sb22*pp + Sb22*sigmaT + Sb22*tns + pp*tns - sigmaT*tns - tns**2)/(2.0*Sb00*nu*cos(2.0*theta) + 4.0*Sb01*nu*sin(2.0*theta) - 2.0*Sb11*nu*cos(2.0*theta) - Sb22 + tns)]
"""


"""
Copyright (c) 2024-2025 ROCK LAB PRIVATE LIMITED
This file is part of "Stresslog" project and is released under the 
GNU Affero General Public License v3.0 (AGPL-3.0)
See the GNU Affero General Public License for more details: <https://www.gnu.org/licenses/agpl-3.0.html>
"""

import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
import matplotlib.colors as mcolors
from PIL import Image

import plotly.tools as tls
import plotly.io as pio

import os

user_home = os.path.expanduser("~/Documents")

def plot_logs_labels(data, styles, points=None, pointstyles=None, y_min=None, y_max=None, width=15, height=10, label_height=20, dpi=100, output_dir = os.path.join(user_home, "Stresslog_plots"), to_plotly=False, title=None, details=None, display=False):
    """
    Wrapper function that calls plot_logs three times to generate the main plot and label plots.
    
    Parameters:
    - Same as plot_logs function, except plot_labels is not included as it's handled internally.
    
    Returns:
    - fig, axes: The figure and axes objects of the main plot.
    """
    
    if details is None:
        details={"unit":"metres","type":"TVD","reference":"KB/DF","KB":0,"GL":0}
    
    labelmax = int(y_max) #2600
    #print(labelmax)
    # Generate simulated data for label plots
    simulated_depth = np.array([labelmax])
    simulated_data = pd.DataFrame({key: np.random.rand(1) for key in data.columns}, index=simulated_depth)
    
    # Call plot_logs for bottom labels
    plot_logs(simulated_data, styles, None, None, labelmax+1, labelmax+2, 
              plot_labels=True, width=width, height=3, label_height=98, 
              dpi=dpi, output_dir=output_dir,to_plotly=to_plotly)
    
    # Call plot_logs for top labels
    plot_logs(simulated_data, styles, None, None, labelmax+1, labelmax, 
              plot_labels=True, width=width, height=3, label_height=98, 
              dpi=dpi, output_dir=output_dir,to_plotly=to_plotly)
    # Call plot_logs for the main plot without labels
    fig, axes = plot_logs(data, styles, points, pointstyles, y_min, y_max, 
                          plot_labels=False, width=width,height=height, label_height=label_height, 
                          dpi=dpi,to_plotly=to_plotly)
    plotfile = os.path.join(output_dir,"PlotFigure.png")
    if title is not None:
        fig.suptitle('Wellbore: ' + title, fontsize=18, y=0.91)
        #fig.text(0.50, 0.89, 'Wellbore: ' + title, fontsize=18, horizontalalignment='center')
        fig.text(0.05, 0.91, 'Depth in '+details["unit"]+", "+details["type"], fontsize=9, horizontalalignment='left')
        fig.text(0.05, 0.89, 'Depth Reference: '+details["reference"], fontsize=9, horizontalalignment='left')
        
        fig.text(0.95, 0.91, 'KB: '+str(details["KB"])+' '+details["unit"]+' above MSL', fontsize=9, horizontalalignment='right')
        fig.text(0.95, 0.89, 'GL: '+str(-(details["GL"]))+' '+details["unit"]+' below MSL', fontsize=9, horizontalalignment='right') if details["GL"]<0 else fig.text(0.95, 0.89, 'GL: '+str(details["GL"])+' '+details["unit"]+' above MSL', fontsize=9, horizontalalignment='right')
    
    plt.savefig(plotfile,dpi=dpi)
    if display:
        plt.show()
    plt.close()
    choptop(55,0,os.path.join(output_dir,'BottomLabel.png'))
    cutify2(os.path.join(output_dir,'PlotFigure.png'),os.path.join(output_dir,'BottomLabel.png'), os.path.join(output_dir,'WellPlot.png'), 0, 300, 0, 0)
    choptop(200,0,os.path.join(output_dir,'WellPlot.png'))
    if to_plotly:
        plotly_fig = tls.mpl_to_plotly(fig)
        normalization_info = {}
        for col, style in styles.items():
            normalization_info[col] = {
                'type': style['type'],
                'left': style['left'],
                'right': style['right']
            }
        if pointstyles:
            for col, style in pointstyles.items():
                normalization_info[col] = {
                    'type': style['type'],
                    'left': style['left'],
                    'right': style['right']
                }
        # Update Plotly figure with custom hover templates
        for trace in plotly_fig.data:
            if 'name' in trace and trace.name in normalization_info:
                col_name = trace.name
                norm_info = normalization_info[col_name]
                
                # Check if it's a line trace (mode should include 'lines')
                if 'lines' in trace.mode:
                    if norm_info['type'] == 'linear':
                        left, right = norm_info['left'], norm_info['right']
                        # Denormalize the data
                        original_data = [x * (right - left) + left for x in trace.x]
                        trace.customdata = original_data
                        hovertemplate = f"{col_name}: " + "%{customdata:.2f}<br>Depth: %{y}<extra></extra>"
                    elif norm_info['type'] == 'log':
                        hovertemplate = f"{col_name}: " + "%{x:.2e}<br>Depth: %{y}<extra></extra>"
                    trace.update(hovertemplate=hovertemplate)
                else:
                    # For scatter plots or unidentified traces, turn off hover
                    trace.update(hoverinfo='skip')
            else:
                # For any trace we can't identify, turn off hover
                trace.update(hoverinfo='skip')
        #plt.close()
        # Fix the x-axis range for both subplots
        plotly_fig.update_xaxes(fixedrange=True)

        # Share the y-axis between subplots
        plotly_fig.update_yaxes(matches='y')

        # Make sure the plot fits the width of the browser
        plotly_fig.update_layout(
            autosize=True,
            width=None,  # Remove the fixed width to make it responsive
            #height=600,  # Set a fixed height if needed, or leave it auto-sized
            margin=dict(l=30, r=30, t=30, b=30)  # Adjust margins as needed
        )
        plotly_fig.update_xaxes(showgrid=True, gridcolor = 'rgba(211, 211, 211, 0.8)', tickcolor='rgba(0, 0, 0, 0)', showticklabels=False)
        plotly_fig.update_yaxes(automargin=False,showgrid=True, gridcolor = 'rgba(211, 211, 211, 0.8)', tickcolor='rgba(0, 0, 0, 0)', tickformat='.0f', linecolor='grey', linewidth=1)
        plotly_fig.update_layout(
            yaxis=dict(showticklabels=True),  # Show labels for the leftmost y-axis
            yaxis2=dict(showticklabels=False),  # Hide labels for the second column
            yaxis3=dict(showticklabels=False),  # Hide labels for the third column
            yaxis4=dict(showticklabels=False),
            yaxis5=dict(showticklabels=False),
            yaxis6=dict(showticklabels=False),
        )
        plotly_fig.update_layout(margin=dict(l=30,r=0,t=0,b=0))
        plotly_fig.update_layout(
            paper_bgcolor='white',  # Background of the entire figure
            plot_bgcolor='white',   # Background of the plot area
        )
        #plotly_fig.update_yaxes(showticklabels=False) # hide all the yticks
        # Save as an interactive HTML file and open it in the browser
        pio.write_json(plotly_fig,os.path.join(output_dir,'plotly.json'))
        #pio.write_html(plotly_fig,'plotly.html')
    return fig, axes

def plot_logs(data, styles, points=None, pointstyles=None, y_min=None, y_max=None, plot_labels=True, width=15,height=10, label_height=20, dpi=100, output_dir = os.path.join(user_home, "Stresslog_plots"), to_plotly=False):
    """
    Plots well log data in tracks and sparse data points.

    Parameters:
    - data: A pandas DataFrame where each column corresponds to a curve to be plotted.
    - styles: A dictionary where keys are column names from the DataFrame and values are dictionaries of style attributes.
              Example: {'lithology1': {"color": "green", "linewidth": 1.5, "style": '.', "track": 0, "left": 0, "right": 150, "type": 'linear'}}
    - points: A pandas DataFrame where each column corresponds to sparse points to be plotted.
    - pointstyles: A dictionary where keys are column names from the points DataFrame and values are dictionaries of style attributes.
              Example: {'ucs': {'color': 'blue', 'pointsize': 10, 'symbol': 'o', 'track': 4, 'left': 0, 'right': 100, 'type': 'linear', 'unit': 'Mpa'}}
    - y_min: Minimum value of the y-axis (depth).
    - y_max: Maximum value of the y-axis (depth).
    - plot_labels: Boolean to specify if the labels should be plotted (default is True).
    - figsize: Tuple specifying the width and height of the figure (default is (15, 10)).
    - label_height: Height of the label rectangles (default is 20).
    - dpi: Dots per inch for the saved figure.
    """
    
    figsize=(width, height)
    if y_max is not None and y_min is not None and y_max > y_min:
        invert_yaxis = True
        pltsign = 1
    else:
        invert_yaxis = False
        pltsign = -1

    if len(data.columns) != len(styles):
        raise ValueError("The number of columns in the data must match the number of entries in the styles dictionary.")

    if points is not None and pointstyles is not None:
        if len(points.columns) != len(pointstyles):
            raise ValueError("The number of columns in the points must match the number of entries in the pointstyles dictionary.")

    n_tracks = max(max(style["track"] for style in styles.values()), 
                   max(pointstyle["track"] for pointstyle in pointstyles.values()) if pointstyles else 0) + 1
    fig, axes = plt.subplots(nrows=1, ncols=n_tracks, figsize=figsize, sharey=True)
    if n_tracks == 1:
        axes = [axes]

    # Function to add labels with style
    def add_label(ax, label, original_x_min, original_x_max, y_offset, color, linewidth, linestyle, units, log_scale=False):
        y_data_coords = y_offset

        if log_scale:
            # For log scale, keep the original label placement
            x_min = original_x_min
            x_max = original_x_max
        else:
            # For linear scale, normalize label placement to 0-1
            x_min = 0
            x_max = 1

        ax.set_axis_off()
        # Plot the representative line with the correct linestyle
        ax.plot([x_min, x_max], [y_data_coords+(0.05*pltsign), y_data_coords+(0.05*pltsign)], color=color, linewidth=linewidth, linestyle=linestyle, clip_on=False, zorder=3)

        # Plot the text above the representative line
        ax.text(x_min, y_data_coords, f"{original_x_min}", ha='left', va='center', fontsize=10, color=color, backgroundcolor='none', clip_on=False, zorder=4)
        if log_scale:
            center = np.sqrt(original_x_min * original_x_max)
        else:
            center = 0.5
        ax.text(center, y_data_coords, label, ha='center', va='center', fontsize=10, color=color, backgroundcolor='none', clip_on=False, zorder=4)
        ax.text(x_max, y_data_coords, f"{original_x_max}", ha='right', va='center', fontsize=10, color=color, backgroundcolor='none', clip_on=False, zorder=4)
        ax.text(center, y_data_coords + (0.09 * pltsign), f"{units}", ha='center', va='center', fontsize=7.5, color=color, backgroundcolor='none', clip_on=False, zorder=4)

    depth = data.index
    if y_min is None:
        y_min = min(depth)
    if y_max is None:
        y_max = max(depth)

    label_offsets = {track: y_min - (y_max - y_min) * (0.01) for i, track in enumerate(range(n_tracks))}
    
    # Dictionary to store normalization information
    #normalization_info = {}
    
    # Plotting data
    for col in data.columns:
        track = styles[col]["track"]
        ax = axes[track]
        x_data = data[col]
        style = styles[col]

        original_left = style["left"]
        original_right = style["right"]

        if style["type"] == 'log':
            ax.set_xscale('log')
            x_plot_data = x_data
            x_min = original_left
            x_max = original_right
            #normalization_info[col] = {'type': 'log', 'left': original_left, 'right': original_right}
        else:
            x_plot_data = (x_data - original_left) / (original_right - original_left)
            x_min = 0
            x_max = 1
            #normalization_info[col] = {'type': 'linear', 'left': original_left, 'right': original_right}

        ax.plot(x_plot_data, depth, label=col, color=style["color"], linewidth=style["linewidth"], linestyle=style["style"])
        # Add fill logic
        if style.get("fill") == "left":
            ax.fill_betweenx(depth, x_min, x_plot_data, color=style["color"], alpha=0.3)
        if style.get("fill") == "right":
            ax.fill_betweenx(depth, x_plot_data, x_max, color=style["color"], alpha=0.3)
        if style.get("fill_between"):
            ref_col = style["fill_between"]["reference"]
            ref_data = data[ref_col]
            if styles[ref_col]["type"] == 'log':
                ref_plot_data = ref_data
            else:
                ref_plot_data = (ref_data - styles[ref_col]["left"]) / (styles[ref_col]["right"] - styles[ref_col]["left"])
            
            fill_colors = style["fill_between"]["colors"]
            
            # Fill above reference line
            ax.fill_betweenx(depth, x_plot_data, ref_plot_data, where=x_plot_data >= ref_plot_data, facecolor=fill_colors[0], alpha=0.3)
            
            if "cmap" in style["fill_between"]:
                cmap_name = style["fill_between"]["cmap"]
                cmap = plt.get_cmap(cmap_name)
                
                colorlog = style["fill_between"]["colorlog"]
                cutoffs = style["fill_between"]["cutoffs"]
                
                # Normalize the colorlog data
                norm = mcolors.Normalize(vmin=min(cutoffs), vmax=max(cutoffs))
                
                # Generate colors based on the normalized values
                color_values = cmap(norm(data[colorlog]))
                
                # Fill below reference line with respective colors
                for color in np.unique(color_values, axis=0):
                    mask = (color_values == color).all(axis=-1)
                    ax.fill_betweenx(depth, x_plot_data, ref_plot_data, where=(x_plot_data < ref_plot_data) & mask, facecolor=color, alpha=1)
            elif "colorlog" in style["fill_between"]:
                colorlog = style["fill_between"]["colorlog"]
                cutoffs = style["fill_between"]["cutoffs"]
                cutoff_colors = style["fill_between"]["fillcolors"]
                
                colors = np.full(data[colorlog].shape, 'orange')  # Default color if no cutoff is matched
                for i, cutoff in enumerate(cutoffs):
                    color = cutoff_colors[i]
                    if i == 0:
                        colors = np.where(data[colorlog] < cutoff, color, colors)
                    else:
                        colors = np.where((data[colorlog] < cutoff) & (data[colorlog] >= cutoffs[i-1]), color, colors)
                
                # Fill below reference line with respective colors
                for color in np.unique(colors):
                    mask = colors == color
                    ax.fill_betweenx(depth, x_plot_data, ref_plot_data, where=(x_plot_data < ref_plot_data) & mask, facecolor=color, alpha=1)
            else:
                # Fallback fill behavior if neither "cmap" nor "colorlog" is present
                ax.fill_betweenx(depth, x_plot_data, ref_plot_data, where=x_plot_data < ref_plot_data, facecolor=fill_colors[1], alpha=0.3)                
        if style["type"] == 'log':
            ax.set_xlim(original_left, original_right)
            ax.set_xticks(np.geomspace(original_left, original_right, num=5))
        else:
            ax.set_xlim(0, 1)
            ax.set_xticks(np.linspace(0, 1, num=5))

        ax.set_ylim(y_min, y_max)
        if invert_yaxis:
            ax.invert_yaxis()

        ax.set_xticklabels([])  # Hide tick labels
        ax.set_xlabel("")
        ax.grid(True, which='both', linestyle='--', linewidth=0.5)
        if plot_labels:
            add_label(ax, col, original_left, original_right, label_offsets[track], style["color"], style["linewidth"], style["style"], style["unit"], log_scale=(style["type"] == 'log'))
            label_offsets[track] -= -1.5 * (y_max - y_min) * (label_height / 1000)
            ax.spines['left'].set_color('white')
            ax.spines['top'].set_color('white')
            ax.spines['bottom'].set_color('white')
            ax.spines['right'].set_color('white')
            ax.yaxis.label.set_color('white')
            ax.tick_params(axis='y', colors='white')

    # Plotting points
    if points is not None and pointstyles is not None:
        for col in points.columns:
            track = pointstyles[col]["track"]
            ax = axes[track]
            y_points = points[col].dropna().index
            x_points = points[col].dropna().values
            style = pointstyles[col]

            if style["type"] == 'log':
                ax.set_xscale('log')
                x_plot_points = x_points
                x_min = style["left"]
                x_max = style["right"]
                #normalization_info[col] = {'type': 'log', 'left': style["left"], 'right': style["right"]}
            else:
                x_plot_points = (x_points - style["left"]) / (style["right"] - style["left"])
                x_min = 0
                x_max = 1
                #normalization_info[col] = {'type': 'linear', 'left': style["left"], 'right': style["right"]}


            ax.scatter(x_plot_points, y_points, color=style["color"], s=style["pointsize"], marker=style["symbol"], zorder=5)
            
            if style.get("uptosurface", False):
                ax.vlines(x_plot_points, y_points, y_max, color=style["color"], linestyles='-', zorder=4)

            if style["type"] == 'log':
                ax.set_xlim(style["left"], style["right"])
                ax.set_xticks(np.geomspace(style["left"], style["right"], num=5))
            else:
                ax.set_xlim(0, 1)
                ax.set_xticks(np.linspace(0, 1, num=5))

            if plot_labels:
                add_label(ax, col, style["left"], style["right"], label_offsets[track], style["color"], 1, "-", style["unit"], log_scale=(style["type"] == 'log'))
                label_offsets[track] -= -1.5 * (y_max - y_min) * (label_height / 1000)

    # Set common y-axis label
    fig.subplots_adjust(left=0.05, right=0.95, wspace=0.025)
    
    if plot_labels:
        if pltsign > 0:
            #plt.tight_layout()
            plt.savefig(os.path.join(output_dir,"BottomLabel.png"), dpi=dpi)
            plt.close()
            return
        else:
            plt.savefig(os.path.join(output_dir,'TopLabel.png'), dpi=dpi)
            if to_plotly:
                plotly_fig = tls.mpl_to_plotly(fig)
                #plt.tight_layout()
                plotly_fig.update_layout(
                    autosize=True,
                    width=None,  # Remove the fixed width to make it responsive
                    #height=600,  # Set a fixed height if needed, or leave it auto-sized
                    margin=dict(l=30, r=0, t=0, b=20)  # Adjust margins as needed
                )
                plotly_fig.update_layout(
                    paper_bgcolor='white',  # Background of the entire figure
                    plot_bgcolor='white',   # Background of the plot area
                )
                plotly_fig.update_xaxes(showgrid=False, tickcolor='rgba(0, 0, 0, 0)',showticklabels=False)
                plotly_fig.update_yaxes(automargin=False,showgrid=False, tickcolor='rgba(0, 0, 0, 0)',tickformat='.0f',tickfont=dict(color='rgba(0, 0, 0, 0)'), linecolor='grey', linewidth=1)
                plotly_fig.update_layout(
                    yaxis=dict(showticklabels=True),  # Show labels for the leftmost y-axis
                    yaxis2=dict(showticklabels=False),  # Hide labels for the second column
                    yaxis3=dict(showticklabels=False),# Hide labels for the third column
                    yaxis4=dict(showticklabels=False),
                    yaxis5=dict(showticklabels=False),
                    yaxis6=dict(showticklabels=False),
                )
                plotly_fig.update_layout(margin=dict(l=50,r=50,t=0,b=0))
                pio.write_json(plotly_fig,os.path.join(output_dir,"TopPlotly.json"))
                #plotly_fig.write_image("TopPlotly.png")
            plt.close()
            return
    #print(normalization_info)
    return fig, axes
    


# Define functions to crop and combine images

def choptop(xx,yy,nm='plot1.png'):
    # Open the image
    img = Image.open(nm)

    # Get the original dimensions
    width, height = img.size

    # Define the number of pixels to crop from the top and bottom
    crop_top = xx  # Replace with your value for 'x'
    crop_bottom = yy  # Replace with your value for 'y'

    # Calculate the cropping box: (left, upper, right, lower)
    crop_box = (0, crop_top, width, height - crop_bottom)

    # Crop the image
    cropped_img = img.crop(crop_box)

    # Save the cropped image, overwriting the original
    cropped_img.save(nm)

    # Optionally, show the cropped image to verify the result
    #cropped_img.show()
    


def chopleft(xx, yy, nm='combined_plot.png'):
    
    # Open the image
    img = Image.open(nm)

    # Get the original dimensions
    width, height = img.size

    # Define the number of pixels to crop from the left and right
    crop_left = xx  # Number of pixels to crop from the left
    crop_right = yy  # Number of pixels to crop from the right

    # Calculate the cropping box: (left, upper, right, lower)
    crop_box = (crop_left, 0, width - crop_right, height)

    # Crop the image
    cropped_img = img.crop(crop_box)

    # Save the cropped image, overwriting the original
    cropped_img.save(nm)

    # Optionally, show the cropped image to verify the result
    # cropped_img.show()


    

#chopleft(50,50)
def joinery(n1,n2,n3,x1,x2):
    choptop(x1,x2,n1) #119,109
    # Open the images
    from PIL import Image
    img2 = Image.open(n1)
    img1 = Image.open(n2)

    # Get the width and height of the images
    width1, height1 = img1.size
    width2, height2 = img2.size

    # Create a new image with the combined height of both images
    combined_img = Image.new('RGB', (max(width1, width2), height1 + height2))

    # Paste the images into the new image
    combined_img.paste(img1, (0, 0))
    combined_img.paste(img2, (0, height1))

    # Save the combined image
    combined_img.save(n3)

def joinery2(n1,n2,n3,x1,x2):
    choptop(x1,x2,n2) #119,109
    # Open the images
    from PIL import Image
    img2 = Image.open(n1)
    img1 = Image.open(n2)

    # Get the width and height of the images
    width1, height1 = img1.size
    width2, height2 = img2.size

    # Create a new image with the combined height of both images
    combined_img = Image.new('RGB', (max(width1, width2), height1 + height2))

    # Paste the images into the new image
    combined_img.paste(img1, (0, 0))
    combined_img.paste(img2, (0, height1))

    # Save the combined image
    combined_img.save(n3)


def cutify(n1,n2,n3,x1,x2,x3,x4):
    joinery(n1,n2,n3,x1,x2)
    # Example usage
    chopleft(x3, x4, n3)#120,120
    # Display the combined image
    img1 = Image.open(n3)
    img1.save(n3)
    #img1.show()
    
def cutify2(n1,n2,n3,x1,x2,x3,x4):
    joinery2(n2,n1,n3,x1,x2)
    # Example usage
    chopleft(x3, x4, n3)#120,120
    # Display the combined image
    img1 = Image.open(n3)
    img1.save(n3)
    #img1.show()

def chopify(n1,x1,x2,x3,x4):
    choptop(x1,x2,n1) #119,109
    # Example usage
    chopleft(x3, x4, n1)#120,120
    # Display the combined image
    img1 = Image.open(n1)
    img1.save(n1)
    #img1.show()

#cutify2('plot.png','BottomLabel.png','combined.png',119,109,120,120)
"""
# Example usage of the modified function
data = pd.DataFrame({
    'log1': np.random.random(100) * 150,
    'log2': np.random.random(100) * 200,
}, index=np.linspace(0, 1000, 100))

styles = {
    'log1': {"color": "green", "linewidth": 1.5, "style": '-', "track": 0, "left": 0, "right": 150, "type": 'linear', "unit": "m/s"},
    'log2': {"color": "blue", "linewidth": 1.5, "style": '-', "track": 1, "left": 0, "right": 200, "type": 'linear', "unit": "m/s"},
}

points = pd.DataFrame({
    'point1': np.random.random(10) * 100,
    'point2': np.random.random(10) * 50,
}, index=np.linspace(0, 1000, 10))

pointstyles = {
    'point1': {'color': 'red', 'pointsize': 50, 'symbol': 'o', 'track': 0, 'left': 0, 'right': 100, 'type': 'linear', 'unit': "Mpa", 'uptosurface': True},
    'point2': {'color': 'purple', 'pointsize': 50, 'symbol': 'o', 'track': 1, 'left': 0, 'right': 50, 'type': 'linear', 'unit': "Mpa"},
}

x = plot_logs(data, styles, points, pointstyles, y_min=0, y_max=1000)
"""



import os
import sys

# Add the parent directory to the Python path
sys.path.insert(0, os.path.abspath('../../'))
# Configuration file for the Sphinx documentation builder.
#
# For the full list of built-in configuration values, see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Project information -----------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#project-information

project = 'stresslog'
copyright = 'ROCK LAB PRIVATE LIMITED'
author = 'Arkadeep Ghosh'
release = '1.6.16'

# -- General configuration ---------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#general-configuration

extensions = [
    'sphinx.ext.autodoc',        # Automatically document from docstrings
    'sphinx.ext.napoleon',       # Support for NumPy and Google-style docstrings
    #'sphinx.ext.viewcode',       # Add links to source code
    ]


templates_path = ['_templates']
exclude_patterns = []

# Autodoc settings
autodoc_preserve_defaults = True  # Shows string literals instead of evaluated values
autodoc_default_options = {
    'members': True,
    'undoc-members': False,       # Exclude undocumented members
    'private-members': False,     # Exclude private members
    'special-members': False,     # Exclude special methods like __init__
    'inherited-members': False,   # Exclude inherited methods
    'imported-members': False,    # Exclude members not defined in the module
}
autodoc_inherit_docstrings = False  # Avoid inheriting docstrings from parents



# -- Options for HTML output -------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#options-for-html-output
html_theme = 'furo'



"""
Copyright (c) 2024-2025 ROCK LAB PRIVATE LIMITED
This file is part of "Stresslog" project and is released under the 
GNU Affero General Public License v3.0 (AGPL-3.0)
See the GNU Affero General Public License for more details: <https://www.gnu.org/licenses/agpl-3.0.html>
"""

import matplotlib.pyplot as plt3
from matplotlib.patches import Polygon
import numpy as np
import math

Sv = 48
Pp = 19
bhp = 22
UCS = 46
PhiB = 45
mu = 0.6

def drawSP(Sv,Pp,bhp,shmin,UCS = 0,phi = 0, flag = 0,mu = 0.65,nu=0.25,bt=0,ym=0,delT=0,PhiBr=15,biot=1,path=None,debug=False, display=False):
    """Draw a stress polygon visualization for wellbore stability analysis.

    Parameters
    ----------
    Sv : float
        Vertical stress in MPa
    Pp : float
        Pore pressure in MPa
    bhp : float
        Bottom hole pressure in MPa
    shmin : float
        Minimum horizontal stress in MPa
    UCS : float, optional
        Unconfined compressive strength in MPa, by default 0
    phi : float, optional
        Internal friction angle in radians, by default 0
    flag : int, optional
        Condition flag for stress calculation:
        0 - No image log exists
        1 - Image log exists, no breakouts or tensile fractures observed
        2 - Breakouts observed
        3 - Tensile fractures observed
        4 - Both breakouts and tensile fractures observed
        By default 0
    mu : float, optional
        Friction coefficient, by default 0.65
    nu : float, optional
        Poisson's ratio, by default 0.25
    bt : float, optional
        Linear thermal expansion coefficient, by default 0
    ym : float, optional
        Young's modulus in MPa, by default 0
    delT : float, optional
        Temperature difference in degrees Celsius, by default 0
    PhiBr : float, optional
        Breakout width in degrees, by default 15
    biot : float, optional
        Biot's poroelastic constant, by default 1
    path : str, optional
        File path to save the plot, by default None

    Returns
    -------
    matplotlib.pyplot
        If path is None, returns the matplotlib plot object
    None
        If path is provided, saves the plot to the specified path

    Notes
    -----
    The units to the function are not strictly in MPa, other units will
    work just fine, as long as all units are compatible with each other
    (for example Sv, Pp, bhp and ucs can all be in psi and the plot will
    also be in psi)
    This function creates a stress polygon visualization that includes:
    - Stress regime domains (Normal, Strike-slip, Reverse)
    - Breakout analysis with UCS variations
    - Drilling-induced tensile fracture (DITF) analysis
    - Stress bounds and interpolated values
    """
    
    #mu = (1-(2*nu))/(2*((nu*(1-nu))**0.5))
    
    #PhiBr = 15
    #biot = 1
    maxSH = 0
    minSH = 0
    midSH = 0
    sigmaV = Sv-Pp
    sigmahmin = shmin-Pp
    ufac = ((((mu**2)+1)**0.5)+mu)**2
    print("Mu factor: ",ufac) if debug else None

    ShmP = ((Sv-Pp)/ufac)+Pp
    SHMP = ((Sv-Pp)*ufac)+Pp
    print("Corners: ",ShmP,SHMP) if debug else None


    maxSt = 1.02*SHMP
    #minSt = 0.98*ShmP
    
    #maxSt = 200
    minSt = 0

    fig,ax = plt3.subplots()
    ax.axis([minSt,maxSt,minSt,maxSt])
    limit = np.array([(0,0),(maxSt,maxSt), (maxSt,0)])
    LM =  Polygon(limit, fill=False ,hatch='\\')
    ax.add_patch(LM)

    X = ShmP
    Y = SHMP

    NNcorners = np.array([(Sv,Sv),(X,Sv),(X,X)])
    SScorners = np.array([(Sv,Sv),(X,Sv),(Sv,Y)])
    RRcorners = np.array([(Sv,Sv),(Sv,Y),(Y,Y)])

    StrikeSlipE = Polygon(SScorners, fill=False)
    NormalE =  Polygon(NNcorners, fill=False)
    ReverseE =  Polygon(RRcorners, fill=False)

    StrikeSlip = Polygon(SScorners, color='blue', alpha=0.05)
    Normal =  Polygon(NNcorners, color='green', alpha = 0.05)
    Reverse =  Polygon(RRcorners, color='red', alpha = 0.05)

    ax.add_patch(StrikeSlip)
    ax.add_patch(Normal)
    ax.add_patch(Reverse)
    ax.add_patch(StrikeSlipE)
    ax.add_patch(NormalE)
    ax.add_patch(ReverseE)

    #UCS = 46
    UCShigh = UCS + (0.2*UCS)
    UCSlow = UCS - (0.2*UCS)
    Shm = ShmP
    Shm2 = SHMP
    #PhiB = 0.1 #degrees
    PhiBr = math.radians(PhiBr)
    TwoCosPhiB = 2*(math.cos((math.pi)-(PhiBr)))
    print("TwoCosPhiB: ",TwoCosPhiB) if debug else None
    #ShmP = UCS
    print("Phi = ",math.degrees(phi)) if debug else None
    twocos2Beta = 2 * (math.cos(PhiBr))
    q = (1+np.sin(phi))/(1-np.sin(phi)) #Zhang 6.70
    
    SHM1 = ((UCS - (biot*(q-1)*Pp)) + ((q+1)*bhp) - ((1-twocos2Beta)*Shm))/(1+twocos2Beta)
    SHM2 = ((UCS - (biot*(q-1)*Pp)) + ((q+1)*bhp) - ((1-twocos2Beta)*Shm2))/(1+twocos2Beta)
    SHM1H = ((UCShigh - (biot*(q-1)*Pp)) + ((q+1)*bhp) - ((1-twocos2Beta)*Shm))/(1+twocos2Beta)
    SHM2H = ((UCShigh - (biot*(q-1)*Pp)) + ((q+1)*bhp) - ((1-twocos2Beta)*Shm2))/(1+twocos2Beta)
    SHM1L = ((UCSlow - (biot*(q-1)*Pp)) + ((q+1)*bhp) - ((1-twocos2Beta)*Shm))/(1+twocos2Beta)
    SHM2L = ((UCSlow - (biot*(q-1)*Pp)) + ((q+1)*bhp) - ((1-twocos2Beta)*Shm2))/(1+twocos2Beta)
    
    """
    SHM1 = ((UCS + (2*Pp) + (bhp-Pp)) - ((Shm)*(1+TwoCosPhiB)))/(1-TwoCosPhiB)
    SHM2 = ((UCS + (2*Pp) + (bhp-Pp)) - ((Shm2)*(1+TwoCosPhiB)))/(1-TwoCosPhiB)
    
    SHM1H = ((UCShigh + (2*Pp) + (bhp-Pp)) - ((Shm)*(1+TwoCosPhiB)))/(1-TwoCosPhiB)
    SHM2H = ((UCShigh + (2*Pp) + (bhp-Pp)) - ((Shm2)*(1+TwoCosPhiB)))/(1-TwoCosPhiB)
    SHM1L = ((UCSlow + (2*Pp) + (bhp-Pp)) - ((Shm)*(1+TwoCosPhiB)))/(1-TwoCosPhiB)
    SHM2L = ((UCSlow + (2*Pp) + (bhp-Pp)) - ((Shm2)*(1+TwoCosPhiB)))/(1-TwoCosPhiB)
    """
    #print(SHM1)
    br1 = np.array([(Shm,SHM1),(Shm2,SHM2)])
    br2 = np.array([(Shm,SHM1H),(Shm2,SHM2H)])
    br3 = np.array([(Shm,SHM1L),(Shm2,SHM2L)])
    
    lowerulow = [Shm,SHM1H]
    upperulow = [Shm2,SHM2H]
    loweruhigh = [Shm,SHM1L]
    upperuhigh = [Shm2,SHM2L]
    lowerucs = [Shm,SHM1]
    upperucs = [Shm2,SHM2]
    
    #print(br1)
    Breakout2 =  Polygon(br2, color='red', label = "UCS- "+str(round(UCShigh))+"MPa")
    Breakout1 =  Polygon(br1, color='green', label = "UCS- "+str(round(UCS))+"MPa")
    Breakout3 =  Polygon(br3, color='blue', label = "UCS- "+str(round(UCSlow))+"MPa")
    ax.add_patch(Breakout2)
    ax.add_patch(Breakout1)
    ax.add_patch(Breakout3)
    
    Shm3 = ShmP
    Shm4 = Y
    
    sigmaT = (ym*bt*delT)/(1-nu)
        
    #DITFshmax3 = 3*Shm3 - 2*Pp
    #DITFshmax4 = 3*Shm4 - 2*Pp
    DITFshmax3 = 3*Shm3-bhp-Pp-sigmaT
    DITFshmax4 = 3*Shm4-bhp-Pp-sigmaT
    #DITFshmax3 = (ufac*Shm3) - ((ufac-1)*Pp) - (bhp-Pp) - sigmaT
    #DITFshmax4 = (ufac*Shm4) - ((ufac-1)*Pp) - (bhp-Pp) - sigmaT
    ditf = np.array([(Shm3,DITFshmax3),(Shm4,DITFshmax4)])
    DITF =  Polygon(ditf, color='aqua', label = 'DITF')
    lowerd = [Shm3,DITFshmax3]
    upperd = [Shm4,DITFshmax4]
    
    if(shmin>Sv):
        minSH = Sv
        maxSH = SHMP
        #return [Sv,shmin,(shmin+Sv)/2]
    if shmin > Sv:
        minSH = shmin
        maxSH = SHMP
    else:
        #lower = np.array([ShmP, Sv])
        #upper = np.array([Sv, SHMP])
        y = np.array([Sv,SHMP])
        x = np.array([ShmP,Sv])
        I1 = np.interp(shmin, x, y)
        #print(shmin,Sv,I1)
        minSH = shmin
        maxSH = I1
    
    
        if flag>0.5:
            UCShigh = UCS + (0.2 * UCS)
            UCSlow = UCS - (0.2 * UCS)
            maxSt = 1.1*SHMP
            minSt = 0.90*ShmP
            
            xulow = np.array([Shm,Shm2])
            yulow = np.array([SHM1H,SHM2H])
            xuhigh = np.array([Shm,Shm2])
            yuhigh = np.array([SHM1L,SHM2L])
            xucs = np.array([Shm,Shm2])
            yucs = np.array([SHM1,SHM2])
            xd = np.array([Shm3,Shm4])
            yd = np.array([DITFshmax3,DITFshmax4])
            
            if flag > 0.5 and flag < 1.5: #no breakouts or tensile fractures seen on existing image log
                
                minSH = shmin
                maxSH = np.interp(shmin, xucs, yucs)
            if flag > 1.5 and flag <2.5: #breakout observed on image log
                minSH = np.interp(shmin, xulow, yulow)
                maxSH = np.interp(shmin, xuhigh, yuhigh)
            if flag > 2.5 and flag < 3.5: #tensile fractures observed on image log
                minSH = np.interp(shmin, xucs, yucs)
                maxSH = np.interp(shmin, xd, yd)
            if flag>3.5:
                maxSH = np.interp(shmin, xd, yd)
                minSH = np.interp(shmin, xucs, yucs)
    midSH = (minSH + maxSH) / 2
    print([maxSH,minSH,shmin,Sv]) if debug else None
    print("DITF :",ditf) if debug else None
    ax.add_patch(DITF)
    # Draw a vertical purple line for Shmin
    ax.plot([shmin, shmin], [minSH, maxSH], color='purple', linewidth=1)
    ax.hlines(y=minSH, xmin=0, xmax=shmin, colors='black', linestyles='dotted', linewidth=0.5)
    ax.hlines(y=maxSH, xmin=0, xmax=shmin, colors='black', linestyles='dotted', linewidth=0.5)
    ax.hlines(y=midSH, xmin=0, xmax=shmin, colors='black', linestyles='dotted', linewidth=1)

    # Adjustments to move the annotations further away from the axes to avoid collision with the main axis labels
    # Determine offsets
    x_offset = 0.019 * (ax.get_xlim()[1] - ax.get_xlim()[0])  # 2% of the x-axis range
    y_offset = 0.019 * (ax.get_ylim()[1] - ax.get_ylim()[0])  # 2% of the y-axis range

    # For the x-axis
    #ax.text(round(shmin), ax.get_ylim()[0] - 4 * y_offset, '{:.0f}'.format(shmin), ha='center', va='top', rotation=0)
    ax.text(round(Sv), ax.get_ylim()[0] - 4 * y_offset, '{:.0f}'.format(Sv), ha='center', va='top', rotation=0)

    # For the y-axis
    # Adjust the x coordinate for y-axis labels to move them further from the axis
    if maxSH-minSH>25:
        ax.text(ax.get_xlim()[0] - 5 * x_offset, round(minSH), '{:.0f}'.format(minSH), ha='right', va='center', rotation=90)
        ax.text(ax.get_xlim()[0] - 5 * x_offset, round(midSH), '{:.0f}'.format(midSH), ha='right', va='center', rotation=90)
        ax.text(ax.get_xlim()[0] - 5 * x_offset, round(maxSH), '{:.0f}'.format(maxSH), ha='right', va='center', rotation=90)
    else:
        ax.text(ax.get_xlim()[0] - 5 * x_offset, round(midSH), '{:.0f}'.format(midSH), ha='right', va='center', rotation=90)
    ax.legend(loc='lower right')
    plt3.gca().set_aspect('equal', adjustable='box')
    plt3.title("Stress Polygon Results: Shmin: "+str(round(shmin,1))+"MPa, SV: "+str(round(Sv,1))+"MPa, SHMAX: "+str(round(midSH,2))+"MPa")
    plt3.xlabel("Shmin", labelpad=10)  # Increase labelpad as needed
    plt3.ylabel("SHmax", labelpad=10)  # Increase labelpad as needed
    if path is not None:
        plt3.savefig(path, dpi=600)
        if display:
            plt3.show()
        plt3.clf()
        plt3.close()
    else:
        return plt3

def getSP(Sv,Pp,bhp,shmin,UCS = 0,phi = 0, flag = 0,mu = 0.6,nu=0.25,bt=0,ym=0,delT=0, PhiBr=15, biot=1):
    """Calculate SHmax using stress polygon logic, returns said value without visualisation.

    Parameters
    ----------
    Sv : float
        Vertical stress in MPa
    Pp : float
        Pore pressure in MPa
    bhp : float
        Bottom hole pressure in MPa
    shmin : float
        Minimum horizontal stress in MPa
    UCS : float, optional
        Unconfined compressive strength in MPa, by default 0
    phi : float, optional
        Internal friction angle in radians, by default 0
    flag : int, optional
        Condition flag for stress calculation:
        0 - No image log exists
        1 - Image log exists, no breakouts or tensile fractures observed
        2 - Breakouts observed
        3 - Tensile fractures observed
        4 - Both breakouts and tensile fractures observed
        By default 0
    mu : float, optional
        Friction coefficient, by default 0.6
    nu : float, optional
        Poisson's ratio, by default 0.25
    bt : float, optional
        Linear thermal expansion coefficient, by default 0
    ym : float, optional
        Young's modulus in MPa, by default 0
    delT : float, optional
        Temperature difference in degrees Celsius between hole-wall and circulating fluid, by default 0
    PhiBr : float, optional
        Breakout width in degrees, by default 15
    biot : float, optional
        Biot's poroelastic constant, by default 1
    Returns
    -------
    list
        A list containing three float values:
        - minSH: Minimum Max horizontal stress (SHMax)(MPa)
        - maxSH: Maximum Max horizontal stress (SHMax)(MPa)
        - midSH: Best Estimate Max horizontal stress (SHMAx)(MPa)

    Notes
    -----
    The units to the function are not strictly in MPa, other units will
    work just fine, as long as all units are compatible with each other
    (for example Sv, Pp, bhp and ucs can all be in psi and the returned
    SHMax will also be in psi)
    This function performs the same stress polygon calculations as drawSP
    but returns the numerical results without creating a visualization.
    """
    
    #PhiBr = 15
    #biot = 1
    maxSH = 0
    minSH = 0
    midSH = 0
    sigmaV = Sv-Pp
    sigmahmin = shmin-Pp
    ufac = ((((mu**2)+1)**0.5)+mu)**2
    #print("Mu factor: ",ufac)

    ShmP = ((Sv-Pp)/ufac)+Pp
    SHMP = ((Sv-Pp)*ufac)+Pp
    #print("Corners: ",ShmP,SHMP)
    
    if shmin<ShmP:
        shmin=ShmP
    if shmin>SHMP:
        shmin=SHMP
    #maxSt = 1.02*SHMP
    #minSt = 0.98*ShmP
    
    maxSt = 200
    minSt = 0

    #fig,ax = plt3.subplots()
    #ax.axis([minSt,maxSt,minSt,maxSt])
    limit = np.array([(0,0),(maxSt,maxSt), (maxSt,0)])
    #LM =  Polygon(limit, fill=False ,hatch='\\')
    #ax.add_patch(LM)

    X = ShmP
    Y = SHMP

    NNcorners = np.array([(Sv,Sv),(X,Sv),(X,X)])
    SScorners = np.array([(Sv,Sv),(X,Sv),(Sv,Y)])
    RRcorners = np.array([(Sv,Sv),(Sv,Y),(Y,Y)])

    #StrikeSlipE = Polygon(SScorners, fill=False)
    #NormalE =  Polygon(NNcorners, fill=False)
    #ReverseE =  Polygon(RRcorners, fill=False)

    #StrikeSlip = Polygon(SScorners, color='blue', alpha=0.05)
    #Normal =  Polygon(NNcorners, color='green', alpha = 0.05)
    #Reverse =  Polygon(RRcorners, color='red', alpha = 0.05)

    #ax.add_patch(StrikeSlip)
    #ax.add_patch(Normal)
    #ax.add_patch(Reverse)
    #ax.add_patch(StrikeSlipE)
    #ax.add_patch(NormalE)
    #ax.add_patch(ReverseE)

    #UCS = 46
    UCShigh = UCS + (0.2*UCS)
    UCSlow = UCS - (0.2*UCS)
    Shm = ShmP
    Shm2 = SHMP
    #PhiB = 0.1 #degrees
    PhiBr = math.radians(PhiBr)
    TwoCosPhiB = 2*(math.cos((math.pi)-(PhiBr)))
    #print("TwoCosPhiB: ",TwoCosPhiB)
    #ShmP = UCS
    #print("Phi = ",math.degrees(phi))
    twocos2Beta = 2 * (np.cos(PhiBr))
    q = (1+np.sin(phi))/(1-np.sin(phi)) # zhang 6.70
    
    SHM1 = ((UCS - (biot*(q-1)*Pp)) + ((q+1)*bhp) - ((1-twocos2Beta)*Shm))/(1+twocos2Beta)
    SHM2 = ((UCS - (biot*(q-1)*Pp)) + ((q+1)*bhp) - ((1-twocos2Beta)*Shm2))/(1+twocos2Beta)
    SHM1H = ((UCShigh - (biot*(q-1)*Pp)) + ((q+1)*bhp) - ((1-twocos2Beta)*Shm))/(1+twocos2Beta)
    SHM2H = ((UCShigh - (biot*(q-1)*Pp)) + ((q+1)*bhp) - ((1-twocos2Beta)*Shm2))/(1+twocos2Beta)
    SHM1L = ((UCSlow - (biot*(q-1)*Pp)) + ((q+1)*bhp) - ((1-twocos2Beta)*Shm))/(1+twocos2Beta)
    SHM2L = ((UCSlow - (biot*(q-1)*Pp)) + ((q+1)*bhp) - ((1-twocos2Beta)*Shm2))/(1+twocos2Beta)
    
    """
    SHM1 = ((UCS + (2*Pp) + (bhp-Pp)) - ((Shm)*(1+TwoCosPhiB)))/(1-TwoCosPhiB)
    SHM2 = ((UCS + (2*Pp) + (bhp-Pp)) - ((Shm2)*(1+TwoCosPhiB)))/(1-TwoCosPhiB)
    
    SHM1H = ((UCShigh + (2*Pp) + (bhp-Pp)) - ((Shm)*(1+TwoCosPhiB)))/(1-TwoCosPhiB)
    SHM2H = ((UCShigh + (2*Pp) + (bhp-Pp)) - ((Shm2)*(1+TwoCosPhiB)))/(1-TwoCosPhiB)
    SHM1L = ((UCSlow + (2*Pp) + (bhp-Pp)) - ((Shm)*(1+TwoCosPhiB)))/(1-TwoCosPhiB)
    SHM2L = ((UCSlow + (2*Pp) + (bhp-Pp)) - ((Shm2)*(1+TwoCosPhiB)))/(1-TwoCosPhiB)
    """
    #print(SHM1)
    br1 = np.array([(Shm,SHM1),(Shm2,SHM2)])
    br2 = np.array([(Shm,SHM1H),(Shm2,SHM2H)])
    br3 = np.array([(Shm,SHM1L),(Shm2,SHM2L)])
    
    lowerulow = [Shm,SHM1H]
    upperulow = [Shm2,SHM2H]
    loweruhigh = [Shm,SHM1L]
    upperuhigh = [Shm2,SHM2L]
    lowerucs = [Shm,SHM1]
    upperucs = [Shm2,SHM2]
    
    #print(br1)
    #Breakout2 =  Polygon(br2, color='red', label = "UCS- "+str(round(UCShigh))+"MPa")
    #Breakout1 =  Polygon(br1, color='green', label = "UCS- "+str(round(UCS))+"MPa")
    #Breakout3 =  Polygon(br3, color='blue', label = "UCS- "+str(round(UCSlow))+"MPa")
    #ax.add_patch(Breakout2)
    #ax.add_patch(Breakout1)
    #ax.add_patch(Breakout3)
    
    Shm3 = ShmP
    Shm4 = Y
    
    sigmaT = (ym*bt*delT)/(1-nu)
    
    #DITFshmax3 = 3*Shm3 - 2*Pp
    #DITFshmax4 = 3*Shm4 - 2*Pp
    DITFshmax3 = (ufac*Shm3) - ((ufac-1)*Pp) - (bhp-Pp) - sigmaT
    DITFshmax4 = (ufac*Shm4) - ((ufac-1)*Pp) - (bhp-Pp) - sigmaT
    ditf = np.array([(Shm3,DITFshmax3),(Shm4,DITFshmax4)])
    DITF =  Polygon(ditf, color='aqua', label = 'DITF')
    lowerd = [Shm3,DITFshmax3]
    upperd = [Shm4,DITFshmax4]
    

    if shmin > Sv:
        minSH = shmin
        maxSH = SHMP
    else:
        #lower = np.array([ShmP, Sv])
        #upper = np.array([Sv, SHMP])
        y = np.array([Sv,SHMP])
        x = np.array([ShmP,Sv])
        I1 = np.interp(shmin, x, y)
        #print(shmin,Sv,I1)
        minSH = shmin
        maxSH = I1
    
    
        if flag>0.5:
            UCShigh = UCS + (0.2 * UCS)
            UCSlow = UCS - (0.2 * UCS)
            maxSt = 1.1*SHMP
            minSt = 0.90*ShmP
            
            xulow = np.array([Shm,Shm2])
            yulow = np.array([SHM1H,SHM2H])
            xuhigh = np.array([Shm,Shm2])
            yuhigh = np.array([SHM1L,SHM2L])
            xucs = np.array([Shm,Shm2])
            yucs = np.array([SHM1,SHM2])
            xd = np.array([Shm3,Shm4])
            yd = np.array([DITFshmax3,DITFshmax4])
            
            if flag > 0.5 and flag < 1.5: #no breakouts or tensile fractures seen on existing image log
                
                minSH = shmin
                maxSH = np.interp(shmin, xucs, yucs)
            if flag > 1.5 and flag <2.5: #breakout observed on image log
                minSH = np.interp(shmin, xulow, yulow)
                maxSH = np.interp(shmin, xuhigh, yuhigh)
            if flag > 2.5 and flag < 3.5: #tensile fractures observed on image log
                minSH = np.interp(shmin, xucs, yucs)
                maxSH = np.interp(shmin, xd, yd)
            if flag>3.5:
                maxSH = np.interp(shmin, xd, yd)
                minSH = np.interp(shmin, xucs, yucs)
        if maxSH<minSH:
            maxSH=minSH
    midSH = (minSH + maxSH) / 2

    return [minSH, maxSH, midSH]


#from .BoreStab import draw
#from .BoreStab import drawStab
#import numpy as np
#import math
"""
def getSHMax_optimized(Sv, Pp, bhp, shmin, UCS=0, phi=30 , flag=0, mu=0.6, biot=1):
    
    ThetaB=15
    #biot=1
    
    ufac = ((((mu**2) + 1)**0.5) + mu)**2

    ShmP = ((Sv - Pp) / ufac) + Pp
    SHMP = ((Sv - Pp) * ufac) + Pp

    

    if shmin<ShmP or shmin>SHMP:
        return [np.nan, np.nan, np.nan]
    if shmin > Sv:
        minSH = shmin
        maxSH = SHMP
    else:
        #lower = np.array([ShmP, Sv])
        #upper = np.array([Sv, SHMP])
        y = np.array([Sv,SHMP])
        x = np.array([ShmP,Sv])
        I1 = np.interp(shmin, x, y)
        #print(shmin,Sv,I1)
        minSH = shmin
        maxSH = I1
    
    
        if flag>0.5:
            UCShigh = UCS + (0.2 * UCS)
            UCSlow = UCS - (0.2 * UCS)
            maxSt = 1.1*SHMP
            minSt = 0.90*ShmP
            Shm = ShmP
            Shm2 = SHMP

            PhiBr = math.radians(ThetaB)
            TwoCosPhiB = 2 * (math.cos((math.pi) - (PhiBr)))
            twocos2Beta = 2 * (math.cos((math.pi) - (2*(PhiBr))))
            q = (1+math.sin(phi))/(1-math.sin(phi))
            
            SHM1 = ((UCS - (biot*(q-1)*Pp)) + ((q+1)*bhp) - ((1-twocos2Beta)*Shm))/(1+twocos2Beta)
            SHM2 = ((UCS - (biot*(q-1)*Pp)) + ((q+1)*bhp) - ((1-twocos2Beta)*Shm2))/(1+twocos2Beta)
            SHM1H = ((UCShigh - (biot*(q-1)*Pp)) + ((q+1)*bhp) - ((1-twocos2Beta)*Shm))/(1+twocos2Beta)
            SHM2H = ((UCShigh - (biot*(q-1)*Pp)) + ((q+1)*bhp) - ((1-twocos2Beta)*Shm2))/(1+twocos2Beta)
            SHM1L = ((UCSlow - (biot*(q-1)*Pp)) + ((q+1)*bhp) - ((1-twocos2Beta)*Shm))/(1+twocos2Beta)
            SHM2L = ((UCSlow - (biot*(q-1)*Pp)) + ((q+1)*bhp) - ((1-twocos2Beta)*Shm2))/(1+twocos2Beta)          
            
            """
"""
            #SHM1 = ((UCS + 2 * Pp + (bhp - Pp)) - Shm * (1 + TwoCosPhiB)) / (1 - TwoCosPhiB)
            #SHM2 = ((UCS + 2 * Pp + (bhp - Pp)) - Shm2 * (1 + TwoCosPhiB)) / (1 - TwoCosPhiB)
            #SHM1H = ((UCShigh + 2 * Pp + (bhp - Pp)) - Shm * (1 + TwoCosPhiB)) / (1 - TwoCosPhiB)
            #SHM2H = ((UCShigh + 2 * Pp + (bhp - Pp)) - Shm2 * (1 + TwoCosPhiB)) / (1 - TwoCosPhiB)
            #SHM1L = ((UCSlow + 2 * Pp + (bhp - Pp)) - Shm * (1 + TwoCosPhiB)) / (1 - TwoCosPhiB)
            #SHM2L = ((UCSlow + 2 * Pp + (bhp - Pp)) - Shm2 * (1 + TwoCosPhiB)) / (1 - TwoCosPhiB)
            """
"""
            xulow = np.array([Shm,Shm2])
            yulow = np.array([SHM1H,SHM2H])
            xuhigh = np.array([Shm,Shm2])
            yuhigh = np.array([SHM1L,SHM2L])
            xucs = np.array([Shm,Shm2])
            yucs = np.array([SHM1,SHM2])


            Shm3 = 1
            Shm4 = maxSt

            DITFshmax3 = ufac * Shm3 - (ufac - 1) * Pp - (bhp - Pp)
            DITFshmax4 = ufac * Shm4 - (ufac - 1) * Pp - (bhp - Pp)
            ditf = np.array([(Shm3, DITFshmax3), (Shm4, DITFshmax4)])
            
            xd = np.array([Shm3,Shm4])
            yd = np.array([DITFshmax3,DITFshmax4])
            if flag > 0.5 and flag < 1.5: #no breakouts or tensile fractures seen on existing image log
                minSH = shmin
                maxSH = np.interp(shmin, xucs, yucs)
            if flag > 1.5 and flag <2.5: #breakout observed on image log
                minSH = np.interp(shmin, xulow, yulow)
                maxSH = np.interp(shmin, xuhigh, yuhigh)
            if flag > 2.5 and flag < 3.5: #tensile fractures observed on image log
                minSH = np.interp(shmin, xucs, yucs)
                maxSH = np.interp(shmin, xd, yd)
            if flag>3.5:# Both breakouts and tensile fractures on log
                maxSH = np.interp(shmin, xd, yd)
                minSH = np.interp(shmin, xucs, yucs)
    #if maxSH<minSH:        
    #    maxSH=minSH
    midSH = (minSH + maxSH) / 2
    return [minSH, maxSH, midSH]
"""


"""
Copyright (c) 2024-2025 ROCK LAB PRIVATE LIMITED
This file is part of "Stresslog" project and is released under the 
GNU Affero General Public License v3.0 (AGPL-3.0)
See the GNU Affero General Public License for more details: <https://www.gnu.org/licenses/agpl-3.0.html>
"""

import math
import numpy as np

def get_OBG_pascals_vec(tvd, tvdbgl, tvdmsl, rhogcc, water, wdf, glwd):
    """
    Vectorized calculation of comprehensive overburden stress considering offshore and onshore scenarios.
    
    Parameters:
    tvd (array-like): True Vertical Depth
    tvdbgl (array-like): True Vertical Depth Below Ground Level
    tvdmsl (array-like): True Vertical Depth from Mean Sea Level
    rhogcc (array-like): Density in g/cc
    water (float): Water density
    wdf (float): Water depth factor
    glwd (float): Ground Level Water Depth (negative for offshore)
    
    Returns:
    tuple: Arrays of integrho (Pa), integrhopsift, and ObgTppg
    """
    tvd = np.asarray(tvd)
    tvdbgl = np.asarray(tvdbgl)
    tvdmsl = np.asarray(tvdmsl)
    rhogcc = np.asarray(rhogcc)
    
    n = len(tvd)
    integrho = np.zeros(n)
    integrhopsift = np.zeros(n)
    ObgTppg = np.zeros(n)
    
    maxwaterppg = wdf * 8.34540426515252 * water
    
    # Calculate depth differences
    depth_diff = np.diff(tvdbgl, prepend=tvdbgl[0])
    
    # Offshore case
    if glwd < 0:
        # Mudline mask
        mudline_mask = tvdbgl > 0
        
        # Calculate integrho
        water_pressure = np.cumsum(water * 9806.65 * depth_diff * (~mudline_mask))
        rock_pressure = np.cumsum(rhogcc * 9806.65 * depth_diff * mudline_mask)
        integrho = water_pressure + rock_pressure
        
        # Calculate integrhopsift
        integrhopsift = (integrho * 0.000145038) / tvd
        
        # Calculate ObgTppg
        ObgTppg = np.where(tvdmsl > 0,
                           np.where(mudline_mask,
                                    (maxwaterppg + (np.cumsum(rhogcc * 8.34540426515252 * depth_diff) / tvdbgl)) / tvdmsl,
                                    8.34540426515252 * water),
                           0)  # Set to 0 where tvdmsl <= 0
    else:
        # Onshore case
        onshore_mask = tvdbgl > 0
        
        # Calculate integrho
        integrho = np.cumsum(rhogcc * 9806.65 * depth_diff * onshore_mask)
        
        # Calculate integrhopsift
        integrhopsift = (integrho * 0.000145038) / tvd
        
        # Calculate ObgTppg (Curved Top Obg Gradient)
        ObgTppg = np.where(onshore_mask,
                           (np.cumsum(rhogcc * 8.34540426515252 * depth_diff) / tvdbgl) / tvd,
                           0)  # Set to 0 where tvdbgl <= 0
    
    return integrho, integrhopsift, ObgTppg

def get_PPgrad_Zhang_gcc(ObgTgcc, pn, b, tvdbgl, c, mudline, matrick, deltmu0, dalm, biot=1):
    """Calculate pore pressure gradient using Zhang's method for a single point.

    Parameters
    ----------
    ObgTgcc : float
        Overburden gradient in gcc units
    pn : float
        Normal (Hydrostatic) pressure gradient in g/cc
    b : float
        Compaction coefficient for unloading case
    tvdbgl : float
        True vertical depth below ground level
    c : float
        Compaction coefficient for loading case
    mudline : float
        interval travel time at mudline (uspf)
    matrick : float
        matrix interval travel time (uspf)
    deltmu0 : float
        interval travel time as recorded on P-sonic log at
        the depth considered to be the top of unloading condition
    dalm : float
        travel time as recorded on P-sonic log (uspf)
        for the given depth
    biot : float, optional
        Biot's coefficient, by default 1

    Returns
    -------
    float
        Calculated pore pressure gradient in gcc units

    Notes
    -----
    Implements Zhang's method for calculating pore pressure gradient.
    The calculation varies based on the relationship between b and c coefficients.
    """
    if b>=c:
        numerator = ObgTgcc - ((ObgTgcc-pn)*((math.log((mudline-matrick))-(math.log(dalm-matrick)))/(c*tvdbgl)))
    else:
        numerator = ObgTgcc - ((ObgTgcc - pn) / (b * tvdbgl)) * ((((b - c) / c) * (math.log((mudline - matrick) / (deltmu0 - matrick)))) + (math.log((mudline - matrick) / (dalm - matrick))))
    
    return numerator / biot

def get_PP_grad_Zhang_gcc_vec(ObgTgcc, pn, b, tvdbgl, c, mudline, matrick, deltmu0, dalm, biot=1):
    """
    Calculate pressure gradient using Zhang's method with vectorized inputs.

    Parameters
    ----------
    ObgTgcc : array_like
        Overburden gradient in g/cc
    pn : array_like
        Normal (Hydrostatic) pressure gradient in g/cc
    b : float/array_like
        Compaction coefficient for unloading case
    tvdbgl : array_like
        True vertical depth below ground level in metres
    c : float/array_like
        Compaction coefficient for loading case
    mudline : float/array_like
        interval travel time at mudline (uspf)
    matrick : float/array_like
        interval travel time in matrix (0 porosity case) (uspf)
    deltmu0 : array_like
        Interval travel time at top of unloading condition in uspf
    dalm : array_like
        P-sonic log array in uspf
    biot : array_like, optional
        Biot coefficient, defaults to 1

    Returns
    -------
    ndarray
        Pressure gradient calculated using Zhang's method

    Notes
    -----
    This function applies Zhang's method for pressure gradient calculation
    with support for vectorized operations. All inputs are broadcast to
    compatible shapes before calculation.
    """
    ObgTgcc = np.asarray(ObgTgcc)
    pn = np.asarray(pn)
    b = np.asarray(b)
    tvdbgl = np.asarray(tvdbgl)
    c = np.asarray(c)
    mudline = np.asarray(mudline)
    matrick = np.asarray(matrick)
    deltmu0 = np.asarray(deltmu0)
    dalm = np.asarray(dalm)
    biot = np.asarray(biot)

    # Broadcast arrays to a common shape
    ObgTgcc, pn, b, tvdbgl, c, mudline, matrick, deltmu0, dalm, biot = np.broadcast_arrays(
        ObgTgcc, pn, b, tvdbgl, c, mudline, matrick, deltmu0, dalm, biot
    )

    # Apply the condition element-wise
    numerator = np.where(
        b >= c,
        ObgTgcc - ((ObgTgcc - pn) * (np.log((mudline - matrick)) - np.log(dalm - matrick)) / (c * tvdbgl)),
        ObgTgcc - ((ObgTgcc - pn) / (b * tvdbgl)) * (
            (((b - c) / c) * np.log((mudline - matrick) / (deltmu0 - matrick))) +
            np.log((mudline - matrick) / (dalm - matrick))
        )
    )

    return numerator / biot


def get_PPgrad_Eaton_gcc(ObgTgcc, pn, be, ne, tvdbgl, res0, resdeep, biot=1):
    """
    Calculate pressure gradient using Eaton's method.

    Parameters
    ----------
    ObgTgcc : float
        Overburden gradient in g/cc
    pn : float
        Normal (Hydrostatic) pressure gradient in g/cc
    be : float
        Eaton's parameter b
    ne : float
        Eaton's exponent
    tvdbgl : float
        True vertical depth below ground level
    res0 : float
        Resistivity at mudline
    resdeep : float
        Deep resistivity measurement of the given sample
    biot : float, optional
        Biot coefficient, defaults to 1

    Returns
    -------
    float
        Pressure gradient calculated using Eaton's method

    Notes
    -----
    This is the single-point version of the Eaton pressure gradient calculation.
    For vectorized operations, use get_PPgrad_Eaton_gcc_vec.
    """
    numerator = ObgTgcc - ((ObgTgcc - pn)*((resdeep/(res0*np.exp(be*tvdbgl)))**ne))
    return numerator / biot

def get_PPgrad_Eaton_gcc_vec(ObgTgcc, pn, be, ne, tvdbgl, res0, resdeep, biot=1):
    """
    Calculate pressure gradient using Eaton's method with vectorized inputs.

    Parameters
    ----------
    ObgTgcc : array_like
        Overburden gradient in g/cc
    pn : array_like
        Normal (Hydrostatic) pressure gradient in g/cc
    be : float/array_like
        Eaton's parameter b
    ne : float/array_like
        Eaton's exponent
    tvdbgl : array_like
        True vertical depth below ground level in metres
    res0 : float/array_like
        Resistivity at mudline in ohm.m
    resdeep : array_like
        Deep resistivity measurements log in ohm.m
    biot : float/array_like, optional
        Biot coefficient, defaults to 1

    Returns
    -------
    ndarray
        Pressure gradient calculated using Eaton's method

    Notes
    -----
    Vectorized version of the Eaton pressure gradient calculation.
    All inputs are broadcast to compatible shapes before calculation.
    """
    # Ensure all inputs are numpy arrays
    ObgTgcc = np.asarray(ObgTgcc)
    pn = np.asarray(pn)
    be = np.asarray(be)
    ne = np.asarray(ne)
    tvdbgl = np.asarray(tvdbgl)
    res0 = np.asarray(res0)
    resdeep = np.asarray(resdeep)
    biot = np.asarray(biot)

    # Broadcast scalar values to match the shape of the largest array
    ObgTgcc, pn, be, ne, tvdbgl, res0, resdeep, biot = np.broadcast_arrays(
        ObgTgcc, pn, be, ne, tvdbgl, res0, resdeep, biot
    )

    # Calculate numerator
    numerator = ObgTgcc - ((ObgTgcc - pn) * ((resdeep / (res0 * np.exp(be * tvdbgl))) ** ne))
    return numerator / biot


def get_PPgrad_Dxc_gcc(ObgTgcc, pn, d, nde, tvdbgl, D0, Dxc, biot=1):
    """
    Calculate pressure gradient using d-exponent method.

    Parameters
    ----------
    ObgTgcc : float
        Overburden gradient in g/cc
    pn : float
        Normal pressure in g/cc
    d : float
        d-exponent parameter
    nde : float
        d-exponent power
    tvdbgl : float
        True vertical depth below ground level
    D0 : float
        d-exponent value at surface
    Dxc : float
        Corrected d-exponent
    biot : float, optional
        Biot coefficient, defaults to 1

    Returns
    -------
    float
        Pressure gradient calculated using d-exponent method

    Notes
    -----
    This is the scalar version of the d-exponent pressure gradient calculation.
    For vectorized operations, use get_PPgrad_Dxc_gcc_vec.
    """
    numerator = ObgTgcc - ((ObgTgcc - pn)*((Dxc/(D0*np.exp(d*tvdbgl)))**nde))
    return numerator / biot

def get_PPgrad_Dxc_gcc_vec(ObgTgcc, pn, d, nde, tvdbgl, D0, Dxc, biot=1):
    """
    Calculate pressure gradient using d-exponent method with vectorized inputs.

    Parameters
    ----------
    ObgTgcc : array_like
        Overburden gradient in g/cc
    pn : array_like
        Normal pressure in g/cc
    d : float/array_like
        d-exponent parameter
    nde : float/array_like
        d-exponent power
    tvdbgl : array_like
        True vertical depth below ground level
    D0 : float/array_like
        Reference d-exponent value
    Dxc : array_like
        Corrected d-exponent
    biot : float/array_like, optional
        Biot coefficient, defaults to 1

    Returns
    -------
    ndarray
        Pressure gradient calculated using d-exponent method

    Notes
    -----
    Vectorized version of the d-exponent pressure gradient calculation.
    All inputs are broadcast to compatible shapes before calculation.
    """
    # Ensure all inputs are numpy arrays
    ObgTgcc = np.asarray(ObgTgcc)
    pn = np.asarray(pn)
    d = np.asarray(d)
    nde = np.asarray(nde)
    tvdbgl = np.asarray(tvdbgl)
    D0 = np.asarray(D0)
    Dxc = np.asarray(Dxc)
    biot = np.asarray(biot)

    # Broadcast scalar values to match the shape of the largest array
    ObgTgcc, pn, d, nde, tvdbgl, D0, Dxc, biot = np.broadcast_arrays(
        ObgTgcc, pn, d, nde, tvdbgl, D0, Dxc, biot
    )

    # Calculate numerator
    numerator = ObgTgcc - ((ObgTgcc - pn) * ((Dxc / (D0 * np.exp(d * tvdbgl))) ** nde))
    return numerator / biot

def get_Dxc(ROP,RPM,WOB,BTDIA,ECD,pn):
    """
    Calculate corrected d-exponent.

    Parameters
    ----------
    ROP : float
        Rate of penetration in ft/hr
    RPM : float
        Rotations per minute
    WOB : float
        Weight on bit in lbs
    BTDIA : float
        Bit diameter in inches
    ECD : float
        Equivalent circulating density
    pn : float
        Hydrostatic pressure gradient, in same units as ECD

    Returns
    -------
    float
        Corrected d-exponent if greater than 0.1, otherwise NaN

    Notes
    -----
    This is the scalar version of the corrected d-exponent calculation.
    For vectorized operations, use get_Dxc_vec.
    """
    #units= ROP:ft/hr, WOB:lbs, BTDIA:in, 
    Dxc = (np.log10(ROP/(60*RPM))*pn)/(np.log10((12*WOB)/((10**6)*BTDIA))*ECD)
    return Dxc if Dxc>0.1 else np.nan

def get_Dxc_vec(ROP, RPM, WOB, BTDIA, ECD, pn):
    """
    Calculate corrected d-exponent with vectorized inputs.

    Parameters
    ----------
    ROP : array_like
        Rate of penetration in ft/hr
    RPM : array_like
        Rotations per minute
    WOB : array_like
        Weight on bit in lbs
    BTDIA : array_like
        Bit diameter in inches
    ECD : array_like
        Equivalent circulating density
    pn : array_like
        Hydrostatic pressure gradient, in same units as ECD

    Returns
    -------
    ndarray
        Array of corrected d-exponent values

    Notes
    -----
    Vectorized version of the corrected d-exponent calculation.
    All inputs are broadcast to compatible shapes before calculation.
    """
    # Ensure all inputs are numpy arrays
    ROP = np.asarray(ROP)
    RPM = np.asarray(RPM)
    WOB = np.asarray(WOB)
    BTDIA = np.asarray(BTDIA)
    ECD = np.asarray(ECD)
    pn = np.asarray(pn)

    # Broadcast scalar values to match the shape of the largest array
    ROP, RPM, WOB, BTDIA, ECD, pn = np.broadcast_arrays(ROP, RPM, WOB, BTDIA, ECD, pn)

    # Calculate Dxc
    Dxc = (np.log10(ROP / (60 * RPM)) * pn) / (np.log10((12 * WOB) / (10**6 * BTDIA)) * ECD)
    return np.where(Dxc > 0.1, Dxc, np.nan)


def get_Shmin_grad_Daine_ppg(nu2, ObgTppg, biot, ppgZhang, tecB):
    """
    Scalar calculation of fracture gradient pressure for a single sample.
    
    Parameters:
    nu2 (float): Poisson's ratio
    ObgTppg (float): Overburden gradient in ppg
    biot (float): Biot's coefficient
    ppgZhang (float): Zhang's pore pressure in ppg
    tecB (float): Tectonic factor
    
    Returns:
    float: Fracture gradient pressure in ppg
    """
    return (nu2 / (1 - nu2)) * (ObgTppg - (biot * ppgZhang)) + (biot * ppgZhang) + (tecB * ObgTppg)

def get_Shmin_grad_Daine_ppg_vec(nu2, ObgTppg, biot, ppgZhang, tecB):
    """
    Vectorized calculation of fracture gradient pressure.
    
    Parameters:
    nu2 (array-like): Poisson's ratio
    ObgTppg (array-like): Overburden gradient in ppg
    biot (array-like): Biot's coefficient
    ppgZhang (array-like): Zhang's pore pressure in ppg
    tecB (array-like): Tectonic factor
    
    Returns:
    numpy.ndarray: Fracture gradient pressure in ppg
    """
    # Convert all inputs to numpy arrays
    nu2 = np.asarray(nu2)
    ObgTppg = np.asarray(ObgTppg)
    biot = np.asarray(biot)
    ppgZhang = np.asarray(ppgZhang)
    tecB = np.asarray(tecB)
    
    return (nu2 / (1 - nu2)) * (ObgTppg - (biot * ppgZhang)) + (biot * ppgZhang) + (tecB * ObgTppg)



from PIL import Image
def overlay_images(base_path, overlay_path, output_path, 
                  x_position=0, y_position=0,
                  x_scale=1.0, y_scale=1.0,
                  transparency=0.75,
                  white_threshold=45):  # New parameter for white detection
    """
    Overlay one image on top of another with transformation controls.
    White pixels in the overlay image become completely transparent.
    
    Parameters:
    base_path (str): Path to the base image
    overlay_path (str): Path to the overlay image
    output_path (str): Path where the result will be saved
    x_position (int): X-axis position for overlay (default: 0)
    y_position (int): Y-axis position for overlay (default: 0)
    x_scale (float): X-axis scaling factor (default: 1.0)
    y_scale (float): Y-axis scaling factor (default: 1.0)
    transparency (float): Transparency level 0-1 (default: 0.75)
    white_threshold (int): RGB difference from white to consider transparent (default: 2)
    """
    
    # Open the images
    base_img = Image.open(base_path).convert('RGBA')
    overlay_img = Image.open(overlay_path).convert('RGBA')
    
    # Scale the overlay image
    new_width = int(overlay_img.width * x_scale)
    new_height = int(overlay_img.height * y_scale)
    overlay_img = overlay_img.resize((new_width, new_height), Image.Resampling.LANCZOS)
    
    # Create a new RGBA image with the same size as the base image
    result = Image.new('RGBA', base_img.size, (0, 0, 0, 0))
    
    # Apply transparency to the overlay
    overlay_data = overlay_img.getdata()
    transparent_overlay = []
    
    for item in overlay_data:
        # Check if the pixel is close to white
        is_white = all(abs(255 - x) <= white_threshold for x in item[:3])
        
        if is_white:
            # Make white pixels completely transparent
            transparent_overlay.append((255, 255, 255, 0))
        else:
            # Apply normal transparency to non-white pixels
            new_alpha = int(item[3] * transparency)
            transparent_overlay.append((item[0], item[1], item[2], new_alpha))
    
    overlay_img.putdata(transparent_overlay)
    
    # Paste the base image
    result.paste(base_img, (0, 0))
    
    # Paste the overlay at the specified position
    result.paste(overlay_img, (x_position, y_position), overlay_img)
    
    # Save the result
    result.save(output_path, 'PNG')
    
    return result

# Example usage in Python console:

# Use the function
result = overlay_images(
    'FMI-2630.png',        # Base image path
    'PlotBHI3.png',       # Overlay image path
    'overlay2.png',         # Output path
    x_position=-168,       # Move overlay 100 pixels right
    y_position=-52,        # Move overlay 50 pixels down
    x_scale=0.157,         # Stretch horizontally by 50%
    y_scale=0.091,         # Compress vertically by 20%
    transparency=0.800     # 75% transparency
)


from azure.storage.blob import ContainerClient
import os
from tqdm import tqdm

# use the shared access signature URI from EquiNor data sharing website after signing in.
# To create this token, go to https://data.equinor.com, and log in (either as employee or as b2c) (ensure pop-ups are allowed).
# You can create your account at this stage using your mail ID.
# Then browse to the dataset in question, in this case Northern Lights (https://data.equinor.com/dataset/NorthernLights).
# The token ("shared access signature URI") will be found at the bottom of the page, in the "Data links" section.
# The token's validity is of limited time (a month or so), you can get a new token by following the steps above once the token expires.

# Initialize the ContainerClient
container_client = ContainerClient.from_container_url("your/shared/access/signature/uri/here")

# List of files to download
files_to_download = [
   "31_5-7 Eos/06.Wireline_Log_Data/WL_RAW_AAC-ARLL-CAL-DEN-GR-NEU_RUN6_EWL_2.DLIS",
   "31_5-7 Eos/02.Drilling_and_Completion/CORING_2020-01-14_REPORT_1.PDF",
   "31_5-7 Eos/03.Directional_Surveys/WELLPATH_COMPUTED_1.ASC",
   "31_5-7 Eos/03.Directional_Surveys/WELLPATH_ORIGINAL_SURVEY_POINTS_1.ASC",
   "31_5-7 Eos/11.Core_Data/CORE_CONV_2020-05-25_REPORT_1.PDF",
   "31_5-7 Eos/12.Geology_Data_and_Evaluations/31_5-7_Formation_Tops_FWR_Sept2020.xlsx"
]

# Specify output directory with a separate variable
output_path = "."  # Relative to where the script is run from

# Create output directory if it doesn't exist
if not os.path.exists(output_path):
   os.makedirs(output_path)
   print(f"\nCreated output directory: {output_path}")

# Download each selected file
for blob_path in files_to_download:
   # Extract just the filename for local saving
   local_filename = os.path.join(output_path, os.path.basename(blob_path))

   # Get blob client and properties
   blob_client = container_client.get_blob_client(blob_path)
   properties = blob_client.get_blob_properties()
   file_size = properties.size

   print(f"\nDownloading {os.path.basename(blob_path)} ({file_size/1024/1024:.2f} MB)...")

   # Download with progress bar
   with open(local_filename, "wb") as download_file:
      download_stream = blob_client.download_blob()

      # Using tqdm for progress reporting
      progress_bar = tqdm(total=file_size, unit='B', unit_scale=True)

      # Download in chunks to show progress
      chunk_size = 1024 * 1024  # 1MB chunks
      for chunk in download_stream.chunks():
            download_file.write(chunk)
            progress_bar.update(len(chunk))

      progress_bar.close()

   print(f"Successfully downloaded {os.path.basename(blob_path)}")

print("\nAll files downloaded successfully!")

# Load support data

import pandas as pd

survey = pd.read_csv('https://raw.githubusercontent.com/GeoArkadeep/supporting-data-for-EOS-Northern-Lights/main/Deviation.csv')
print(survey)
formations = pd.read_csv('https://raw.githubusercontent.com/GeoArkadeep/supporting-data-for-EOS-Northern-Lights/main/NorthernLights-31_5-7.csv')
print(formations.head())
print(list(formations))

"""
   Top TVD  Number                  Formation Name  ...  DXP_NCT  DXP_exp  DXP_ML
0      488       1  URU(Upperregionalunconformity)  ...      NaN      NaN     NaN
1      772       2                           Skade  ...      NaN      NaN     NaN
2     1144       3              HordalandGreenClay  ...      NaN      NaN     NaN
3     1442       4                          Balder  ...      NaN      NaN     NaN
4     1530       5                            Sele  ...      NaN      NaN     NaN

[5 rows x 24 columns]
['Top TVD', 'Number', 'Formation Name', 'GR Cut', 'Struc.Top', 'Struc.Bottom', 'CentroidRatio',
'OWC', 'GOC', 'Coeff.Vol.Therm.Exp.', 'SHMax Azim.', 'SVDip', 'SVDipAzim', 'Tectonic Factor',
'InterpretedSH/Sh', 'Biot', 'Dt_NCT', 'Dt_ML', 'Res_NCT', 'Res_Exp', 'Res_ML', 'DXP_NCT', 'DXP_exp', 'DXP_ML']
"""
# The formations data, if provided, must contain 24 columns in this exact order.
# If values are unavailable or we wish to use the defaults/constant values, it is fine to leave them blank

ucs = pd.read_csv('https://raw.githubusercontent.com/GeoArkadeep/supporting-data-for-EOS-Northern-Lights/main/UCSdata.csv')
print(ucs.head())
"""
   2643.08  35
0  2644.02  34
1  2645.02  35
2  2646.25  31
3  2647.50  37
4  2648.55  34
"""
# The UCS data if provided, must be in MPa, with the depths in metres, TVD.

imagelog = pd.read_csv('https://raw.githubusercontent.com/GeoArkadeep/supporting-data-for-EOS-Northern-Lights/main/31_5-7_Image.csv')
#Image log is available and features are visible, so we will use them here.

import stresslog as lst
from welly import Well

alias = {
   "sonic": ["none", "DTC", "DT24", "DTCO", "DT", "AC", "AAC", "DTHM"],
   "ssonic": ["none", "DTSM","DTSH_FINAL"],
   "gr": ["none", "GR", "GRD", "CGR", "GRR", "GRCFM","GR_EDTC"],
   "resdeep": ["none", "HDRS", "LLD", "M2RX", "MLR4C", "RD", "RT90", "RLA1", "RDEP", "RLLD", "RILD", "ILD", "RT_HRLT", "RACELM"],
   "resshal": ["none", "LLS", "HMRS", "M2R1", "RS", "RFOC", "ILM", "RSFL", "RMED", "RACEHM", "RXO_HRLT"],
   "density": ["none", "ZDEN", "RHOB", "RHOZ", "RHO", "DEN", "RHO8", "BDCFM"],
   "neutron": ["none", "CNCF", "NPHI", "NEU", "TNPH", "NPHI_LIM"],
   "pe": ["none", "PEFLA", "PEF8", "PE"]
}

# Load well log data
vertwell = lst.get_well_from_dlis('WL_RAW_AAC-ARLL-CAL-DEN-GR-NEU_RUN6_EWL_2.DLIS', aliases=alias, step=0.147)
# we could have used aliases=None (which is the default) but that would have returned ALL the channels in the dlis creating a huge las file which slows the analysis somewhat.
# also steps less than 0.15m will be reset to 0.15m, this is a protective measure, upsampling data beyond it's natural resolution with this method creates artefacts and other issues. 

# Set up mud KB, GL, BHT and LOT values
attrib = [50, -307, 0, 0, 0, 100, 0, 0]
xlot = [[1.43, 2582.9]]
# Create vertical well model
wellwithoutdeviation = lst.getwelldev(wella=vertwell, deva=None)
# Run initial analysis
output = lst.compute_geomech(
   wellwithoutdeviation,
   attrib=attrib,
   rhoappg=17.33,
   a=0.8,
   lamb=0.00075,
   forms=formations,
   UCSs=ucs,
   writeFile=True,
   user_home="./output",
   offset=91,
   dip_dir=180,
   dip=2,
   doi=2627.5,
   mwvalues=[[1.26, 0.0, 0.0, 0.0, 0.0, 0]],
   plotstart=2560,
   plotend=2660,
   mudtemp=35,
   fracgradvals=xlot,
)

# Let's check the "PlotAll.png" in the output/Stresslog_Plots to see the zobackogram, stability plot, sanding risk plot and synthetic borehole image
# Let's also compare the "PlotBHI.png" to the actual image log of the Northern Lights Eos well

# While the inbuilt plotting tools work, the main output is the dataframe (and the las string generated from the dataframe and other info)

print(output[0])
print(list(output[0]))
"""
                  DEPT  DTCO  ...  Shear_Modulus     Bulk_Modulus
0         0.0000000000   NaN  ...   0.0000000000     0.0000000000
1         0.1470000000   NaN  ...   0.0000000000     0.0000000000
2         0.2940000000   NaN  ...   0.0000000000     0.0000000000
3         0.4410000000   NaN  ...   0.0000000000     0.0000000000
4         0.5880000000   NaN  ...   0.0000000000     0.0000000000
...                ...   ...  ...            ...              ...
19828  2914.7159999987   NaN  ...   2.2818260018  6776.4871193190
19829  2914.8629999987   NaN  ...   2.2820489493  6777.2354994299
19830  2915.0099999987   NaN  ...   2.2822719005  6777.9839018296
19831  2915.1569999987   NaN  ...   2.2824948556  6778.7323265105
19832  2915.3039999987   NaN  ...   0.0000000000     0.0000000000

[19833 rows x 40 columns]
['DEPT', 'DTCO', 'DTSM', 'GR', 'NPHI', 'RLA1', 'RXO_HRLT', 'RHOZ', 'PEFLA', 'MD', 'TVDM', 'INCL', 'AZIM', 'ShaleFlag', 'RHO', 'OBG_AMOCO', 'DTCT', 'PP_GRADIENT', 'SHmin_DAINES', 'SHmin_ZOBACK', 'FracGrad', 'FracPressure', 'GEOPRESSURE', 'SHmin_PRESSURE', 'SHmax_PRESSURE', 'MUD_PRESSURE', 'OVERBURDEN_PRESSURE', 'HYDROSTATIC_PRESSURE', 'MUD_GRADIENT', 'S0_Lal', 'S0_Lal_Phi', 'UCS_horsrud', 'S0 Lal', 'phi_lal', 'phi_lang', 'Poisson_Ratio', 'ML90', 'Youngs_Modulus', 'Shear_Modulus', 'Bulk_Modulus']
"""

print(output[1][:2500])
"""
~Version ---------------------------------------------------
VERS.   2.0 : CWLS log ASCII Standard -VERSION 2.0
WRAP.    NO : One line per depth step
DLM . SPACE : Column Data Section Delimiter
~Well ------------------------------------------------------
STRT.m     0.00000 : 
STOP.m  2915.30400 : 
STEP.m     0.14700 : 
NULL.      -999.25 : Null value
UWI .       31/5-7 : 
WELL.       31/5-7 : 
SRVC. Schlumberger : 
COMP.      Equinor : 
FLD .          Eos : 
~Curve Information -----------------------------------------
DEPT                .m      : 
DTCO                .us/ft  : 
DTSM                .us/ft  : 
GR                  .gAPI   : 
NPHI                .m3/m3  : 
RLA1                .ohm.m  : 
RXO_HRLT            .ohm.m  : 
RHOZ                .g/cm3  : 
PEFLA               .       : 
MD                  .m      : 
TVDM                .m      : 
INCL                .       : 
AZIM                .       : 
ShaleFlag           .       : 
RHO                 .gcc    : 
OBG_AMOCO           .gcc    : 
DTCT                .       : 
PP_GRADIENT         .gcc    : 
SHmin_DAINES        .gcc    : 
SHmin_ZOBACK        .gcc    : 
FracGrad            .gcc    : 
FracPressure        .psi    : 
GEOPRESSURE         .psi    : 
SHmin_PRESSURE      .psi    : 
SHmax_PRESSURE      .psi    : 
MUD_PRESSURE        .psi    : 
OVERBURDEN_PRESSURE .psi    : 
HYDROSTATIC_PRESSURE.psi    : 
MUD_GRADIENT        .gcc    : 
S0_Lal              .       : 
S0_Lal_Phi          .       : 
UCS_horsrud         .mpa    : 
S0 Lal              .mpa    : 
phi_lal             .       : 
phi_lang            .       : 
Poisson_Ratio       .       : 
ML90                .gcc    : 
Youngs_Modulus      .       : 
Shear_Modulus       .       : 
Bulk_Modulus        .       : 
~Params ----------------------------------------------------
SMALL_RING                       .in                                                                                               8.0 : Caliper Calibration Small Ring
LARGE_RING                       .in                                                                                11.999999046325684 : Caliper Calibration Large Ring
NCT_WATER_TEMP                   .degC                                                                              24.444442749023438 : Calibration Tank Water Temperature
GR_JIG_REF                       .gAPI                                                                                           160.0 : Jig minus background reference
"""


# Create deviated well model
wellwithdeviation = lst.getwelldev(wella=vertwell, deva=survey)
# Run analysis with deviation but no stress tensor tilt
output = lst.compute_geomech(
    wellwithdeviation,
    attrib=attrib,
    rhoappg=17.33,
    lamb=0.00075,
    forms=formations,
    UCSs=ucs,
    writeFile=True,
    user_home="./output0",
    offset=91,
    dip_dir=180,
    dip=0,
    doi=2627.5,
    mwvalues=[[1.26, 0.0, 0.0, 0.0, 0.0, 0]],
    plotstart=2560,
    plotend=2660,
    mudtemp=35,
    fracgradvals=xlot
)


output = lst.compute_geomech(
    wellwithdeviation,
    attrib=attrib,
    rhoappg=17.33,
    lamb=0.00075,
    forms=formations,
    UCSs=ucs,
    writeFile=True,
    user_home="./output1",
    offset=91,
    dip_dir=180,
    dip=2,
    doi=2627.5,
    mwvalues=[[1.26, 0.0, 0.0, 0.0, 0.0, 0]],
    plotstart=2560,
    plotend=2660,
    mudtemp=35,
    fracgradvals=xlot
)

# Extract SHmax azimuth from log data
y = lst.get_dlis_data('WL_RAW_AAC-ARLL-CAL-DEN-GR-NEU_RUN6_EWL_2.DLIS')
z = y[0]["FSH_AZIM_OVERALL"]
unwrapped_z = z.where(z >= 0, z + 180)

# Plot the azimuth values
from matplotlib import pyplot as plt
plt.plot(unwrapped_z)
plt.xlabel("Depth (metres)")
plt.ylabel("Fast Shear Azimuth as per log data (degrees)")
plt.savefig('SHmax_Azim.png')


output = lst.compute_geomech(
    wellwithdeviation,
    attrib=attrib,
    rhoappg=17.33,
    lamb=0.00075,
    forms=formations,
    UCSs=ucs,
    writeFile=True,
    user_home="./output2",
    offset=114,
    dip_dir=180,
    dip=2,
    doi=2627.5,
    mwvalues=[[1.26, 0.0, 0.0, 0.0, 0.0, 0]],
    plotstart=2560,
    plotend=2660,
    mudtemp=35,
    fracgradvals=xlot,
    ten_fac=0
)

output = lst.compute_geomech(
    wellwithdeviation,
    attrib=attrib,
    rhoappg=17.33,
    lamb=0.00075,
    forms=formations,
    UCSs=ucs,
    writeFile=True,
    user_home="./output2",
    offset=124,
    dip_dir=180,
    dip=2,
    doi=2627.5,
    mwvalues=[[1.26, 0.0, 0.0, 0.0, 0.0, 0]],
    plotstart=2560,
    plotend=2660,
    mudtemp=35,
    fracgradvals=xlot,
    ten_fac=0
)